@@ -157,7 +157,7 @@ static int _socket_getaddrinfo2(const mp_obj_t host, const mp_obj_t portx, struc
     };
 
     mp_obj_t port = portx;
-    if (MP_OBJ_IS_SMALL_INT(port)) {
+    if (mp_obj_is_small_int(port)) {
         // This is perverse, because lwip_getaddrinfo promptly converts it back to an int, but
         // that's the API we have to work with ...
         port = mp_obj_str_binary_op(MP_BINARY_OP_MODULO, mp_obj_new_str_via_qstr("%s", 2), port);
@@ -178,12 +178,44 @@ static int _socket_getaddrinfo2(const mp_obj_t host, const mp_obj_t portx, struc
     return res;
 }
 
-int _socket_getaddrinfo(const mp_obj_t addrtuple, struct addrinfo **resp) {
-    mp_uint_t len = 0;
+STATIC void _socket_getaddrinfo(const mp_obj_t addrtuple, struct addrinfo **resp) {
     mp_obj_t *elem;
-    mp_obj_get_array(addrtuple, &len, &elem);
-    if (len != 2) return -1;
-    return _socket_getaddrinfo2(elem[0], elem[1], resp);
+    mp_obj_get_array_fixed_n(addrtuple, 2, &elem);
+    int res = _socket_getaddrinfo2(elem[0], elem[1], resp);
+    if (res != 0) {
+        mp_raise_OSError(res);
+    }
+    if (*resp == NULL) {
+        mp_raise_OSError(-2); // name or service not known
+    }
+}
+
+STATIC mp_obj_t socket_make_new(const mp_obj_type_t *type_in, size_t n_args, size_t n_kw, const mp_obj_t *args) {
+    mp_arg_check_num(n_args, n_kw, 0, 3, false);
+
+    socket_obj_t *sock = m_new_obj_with_finaliser(socket_obj_t);
+    sock->base.type = type_in;
+    sock->domain = AF_INET;
+    sock->type = SOCK_STREAM;
+    sock->proto = 0;
+    sock->peer_closed = false;
+    if (n_args > 0) {
+        sock->domain = mp_obj_get_int(args[0]);
+        if (n_args > 1) {
+            sock->type = mp_obj_get_int(args[1]);
+            if (n_args > 2) {
+                sock->proto = mp_obj_get_int(args[2]);
+            }
+        }
+    }
+
+    sock->fd = lwip_socket(sock->domain, sock->type, sock->proto);
+    if (sock->fd < 0) {
+        exception_from_errno(errno);
+    }
+    _socket_settimeout(sock, UINT64_MAX);
+
+    return MP_OBJ_FROM_PTR(sock);
 }
 
 STATIC mp_obj_t socket_bind(const mp_obj_t arg0, const mp_obj_t arg1) {
@@ -221,7 +253,13 @@ STATIC mp_obj_t socket_accept(const mp_obj_t arg0) {
         if (errno != EAGAIN) exception_from_errno(errno);
         check_for_exceptions();
     }
-    if (new_fd < 0) mp_raise_OSError(MP_ETIMEDOUT);
+    if (new_fd < 0) {
+        if (self->retries == 0) {
+            mp_raise_OSError(MP_EAGAIN);
+        } else {
+            mp_raise_OSError(MP_ETIMEDOUT);
+        }
+    }
 
     // create new socket object
     socket_obj_t *sock = m_new_obj_with_finaliser(socket_obj_t);
@@ -339,7 +377,13 @@ void _socket_settimeout(socket_obj_t *sock, uint64_t timeout_ms) {
 STATIC mp_obj_t socket_settimeout(const mp_obj_t arg0, const mp_obj_t arg1) {
     socket_obj_t *self = MP_OBJ_TO_PTR(arg0);
     if (arg1 == mp_const_none) _socket_settimeout(self, UINT64_MAX);
-    else _socket_settimeout(self, mp_obj_get_float(arg1) * 1000L);
+    else {
+        #if MICROPY_PY_BUILTINS_FLOAT
+        _socket_settimeout(self, mp_obj_get_float(arg1) * 1000L);
+        #else
+        _socket_settimeout(self, mp_obj_get_int(arg1) * 1000);
+        #endif
+    }
     return mp_const_none;
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_2(socket_settimeout_obj, socket_settimeout);
@@ -369,9 +413,24 @@ STATIC mp_uint_t _socket_read_data(mp_obj_t self_in, void *buf, size_t size,
 
     // XXX Would be nicer to use RTC to handle timeouts
     for (int i = 0; i <= sock->retries; ++i) {
-        MP_THREAD_GIL_EXIT();
+        // Poll the socket to see if it has waiting data and only release the GIL if it doesn't.
+        // This ensures higher performance in the case of many small reads, eg for readline.
+        bool release_gil;
+        {
+            fd_set rfds;
+            FD_ZERO(&rfds);
+            FD_SET(sock->fd, &rfds);
+            struct timeval timeout = { .tv_sec = 0, .tv_usec = 0 };
+            int r = select(sock->fd + 1, &rfds, NULL, NULL, &timeout);
+            release_gil = r != 1;
+        }
+        if (release_gil) {
+            MP_THREAD_GIL_EXIT();
+        }
         int r = lwip_recvfrom_r(sock->fd, buf, size, 0, from, from_len);
-        MP_THREAD_GIL_ENTER();
+        if (release_gil) {
+            MP_THREAD_GIL_ENTER();
+        }
         if (r == 0) {
             sock->peer_closed = true;
         }
@@ -441,9 +500,9 @@ int _socket_send(socket_obj_t *sock, const char *data, size_t datalen) {
 
 STATIC mp_obj_t socket_send(const mp_obj_t arg0, const mp_obj_t arg1) {
     socket_obj_t *sock = MP_OBJ_TO_PTR(arg0);
-    mp_uint_t datalen;
-    const char *data = mp_obj_str_get_data(arg1, &datalen);
-    int r = _socket_send(sock, data, datalen);
+    mp_buffer_info_t bufinfo;
+    mp_get_buffer_raise(arg1, &bufinfo, MP_BUFFER_READ);
+    int r = _socket_send(sock, bufinfo.buf, bufinfo.len);
     return mp_obj_new_int(r);
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_2(socket_send_obj, socket_send);
@@ -597,37 +656,11 @@ STATIC const mp_stream_p_t socket_stream_p = {
 STATIC const mp_obj_type_t socket_type = {
     { &mp_type_type },
     .name = MP_QSTR_socket,
+    .make_new = socket_make_new,
     .protocol = &socket_stream_p,
     .locals_dict = (mp_obj_t)&socket_locals_dict,
 };
 
-STATIC mp_obj_t get_socket(size_t n_args, const mp_obj_t *args) {
-    socket_obj_t *sock = m_new_obj_with_finaliser(socket_obj_t);
-    sock->base.type = &socket_type;
-    sock->domain = AF_INET;
-    sock->type = SOCK_STREAM;
-    sock->proto = 0;
-    sock->peer_closed = false;
-    if (n_args > 0) {
-        sock->domain = mp_obj_get_int(args[0]);
-        if (n_args > 1) {
-            sock->type = mp_obj_get_int(args[1]);
-            if (n_args > 2) {
-                sock->proto = mp_obj_get_int(args[2]);
-            }
-        }
-    }
-
-    sock->fd = lwip_socket(sock->domain, sock->type, sock->proto);
-    if (sock->fd < 0) {
-        exception_from_errno(errno);
-    }
-    _socket_settimeout(sock, UINT64_MAX);
-
-    return MP_OBJ_FROM_PTR(sock);
-}
-STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(get_socket_obj, 0, 3, get_socket);
-
 STATIC mp_obj_t esp_socket_getaddrinfo(size_t n_args, const mp_obj_t *args) {
     // TODO support additional args beyond the first two
 
@@ -678,7 +711,7 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_0(esp_socket_initialize_obj, esp_socket_initializ
 STATIC const mp_rom_map_elem_t mp_module_socket_globals_table[] = {
     { MP_ROM_QSTR(MP_QSTR___name__), MP_ROM_QSTR(MP_QSTR_usocket) },
     { MP_ROM_QSTR(MP_QSTR___init__), MP_ROM_PTR(&esp_socket_initialize_obj) },
-    { MP_ROM_QSTR(MP_QSTR_socket), MP_ROM_PTR(&get_socket_obj) },
+    { MP_ROM_QSTR(MP_QSTR_socket), MP_ROM_PTR(&socket_type) },
     { MP_ROM_QSTR(MP_QSTR_getaddrinfo), MP_ROM_PTR(&esp_socket_getaddrinfo_obj) },
 
     { MP_ROM_QSTR(MP_QSTR_AF_INET), MP_ROM_INT(AF_INET) },
