@@ -11,19 +11,19 @@
 #include "py/stackctrl.h"
 #include "py/gc.h"
 #include "py/mperrno.h"
+#include "lib/utils/gchelper.h"
 
 #include "tinytest.h"
 #include "tinytest_macros.h"
 
-#define HEAP_SIZE (128 * 1024)
-STATIC void *heap;
+#define HEAP_SIZE (100 * 1024)
 
 #include "genhdr/tests.h"
 
 int main() {
     mp_stack_ctrl_init();
     mp_stack_set_limit(10240);
-    heap = malloc(HEAP_SIZE);
+    static uint32_t heap[HEAP_SIZE / sizeof(uint32_t)];
     upytest_set_heap(heap, (char*)heap + HEAP_SIZE);
     int r = tinytest_main(0, NULL, groups);
     printf("status: %d\n", r);
@@ -34,10 +34,8 @@ void gc_collect(void) {
     gc_collect_start();
 
     // get the registers and the sp
-    jmp_buf env;
-    setjmp(env);
-    volatile mp_uint_t dummy;
-    void *sp = (void*)&dummy;
+    uintptr_t regs[10];
+    uintptr_t sp = gc_helper_get_regs_and_sp(regs);
 
     // trace the stack, including the registers (since they live on the stack in this function)
     gc_collect_root((void**)sp, ((uint32_t)MP_STATE_THREAD(stack_top) - (uint32_t)sp) / sizeof(uint32_t));
