@@ -51,26 +51,45 @@
 ///     val = adc.read_core_vref()      # read MCU VREF
 
 /* ADC defintions */
+
 #if defined(STM32H7)
 #define ADCx                    (ADC3)
+#define PIN_ADC_MASK            PIN_ADC3
+#define pin_adc_table           pin_adc3
 #else
 #define ADCx                    (ADC1)
+#define PIN_ADC_MASK            PIN_ADC1
+#define pin_adc_table           pin_adc1
 #endif
+
 #define ADCx_CLK_ENABLE         __HAL_RCC_ADC1_CLK_ENABLE
 #define ADC_NUM_CHANNELS        (19)
 
-#if defined(STM32F4)
+#if defined(STM32F0)
 
 #define ADC_FIRST_GPIO_CHANNEL  (0)
 #define ADC_LAST_GPIO_CHANNEL   (15)
+#define ADC_SCALE_V             (3.3f)
+#define ADC_CAL_ADDRESS         (0x1ffff7ba)
+#define ADC_CAL1                ((uint16_t*)0x1ffff7b8)
+#define ADC_CAL2                ((uint16_t*)0x1ffff7c2)
+#define ADC_CAL_BITS            (12)
+
+#elif defined(STM32F4)
+
+#define ADC_FIRST_GPIO_CHANNEL  (0)
+#define ADC_LAST_GPIO_CHANNEL   (15)
+#define ADC_SCALE_V             (3.3f)
 #define ADC_CAL_ADDRESS         (0x1fff7a2a)
 #define ADC_CAL1                ((uint16_t*)(ADC_CAL_ADDRESS + 2))
 #define ADC_CAL2                ((uint16_t*)(ADC_CAL_ADDRESS + 4))
+#define ADC_CAL_BITS            (12)
 
 #elif defined(STM32F7)
 
 #define ADC_FIRST_GPIO_CHANNEL  (0)
 #define ADC_LAST_GPIO_CHANNEL   (15)
+#define ADC_SCALE_V             (3.3f)
 #if defined(STM32F722xx) || defined(STM32F723xx) || \
     defined(STM32F732xx) || defined(STM32F733xx)
 #define ADC_CAL_ADDRESS         (0x1ff07a2a)
@@ -80,23 +99,28 @@
 
 #define ADC_CAL1                ((uint16_t*)(ADC_CAL_ADDRESS + 2))
 #define ADC_CAL2                ((uint16_t*)(ADC_CAL_ADDRESS + 4))
+#define ADC_CAL_BITS            (12)
 
 #elif defined(STM32H7)
 
 #define ADC_FIRST_GPIO_CHANNEL  (0)
 #define ADC_LAST_GPIO_CHANNEL   (16)
+#define ADC_SCALE_V             (3.3f)
 #define ADC_CAL_ADDRESS         (0x1FF1E860)
 #define ADC_CAL1                ((uint16_t*)(0x1FF1E820))
 #define ADC_CAL2                ((uint16_t*)(0x1FF1E840))
+#define ADC_CAL_BITS            (16)
 #define ADC_CHANNEL_VBAT        ADC_CHANNEL_VBAT_DIV4
 
 #elif defined(STM32L4)
 
 #define ADC_FIRST_GPIO_CHANNEL  (1)
 #define ADC_LAST_GPIO_CHANNEL   (16)
+#define ADC_SCALE_V             (3.0f)
 #define ADC_CAL_ADDRESS         (0x1fff75aa)
 #define ADC_CAL1                ((uint16_t*)(ADC_CAL_ADDRESS - 2))
 #define ADC_CAL2                ((uint16_t*)(ADC_CAL_ADDRESS + 0x20))
+#define ADC_CAL_BITS            (12)
 
 #else
 
@@ -104,21 +128,26 @@
 
 #endif
 
-#if defined(STM32F405xx) || defined(STM32F415xx) || \
-    defined(STM32F407xx) || defined(STM32F417xx) || \
-    defined(STM32F401xC) || defined(STM32F401xE) || \
-    defined(STM32F411xE)
+#if defined(STM32F091xC)
 #define VBAT_DIV (2)
-#elif defined(STM32F427xx) || defined(STM32F429xx) || \
+#elif defined(STM32F405xx) || defined(STM32F415xx) || \
+      defined(STM32F407xx) || defined(STM32F417xx) || \
+      defined(STM32F401xC) || defined(STM32F401xE)
+#define VBAT_DIV (2)
+#elif defined(STM32F411xE) || defined(STM32F413xx) || \
+      defined(STM32F427xx) || defined(STM32F429xx) || \
       defined(STM32F437xx) || defined(STM32F439xx) || \
-      defined(STM32F722xx) || defined(STM32F723xx) || \
+      defined(STM32F446xx)
+#define VBAT_DIV (4)
+#elif defined(STM32F722xx) || defined(STM32F723xx) || \
       defined(STM32F732xx) || defined(STM32F733xx) || \
-      defined(STM32F746xx) || defined(STM32F767xx) || \
-      defined(STM32F769xx) || defined(STM32F446xx)
+      defined(STM32F746xx) || defined(STM32F765xx) || \
+      defined(STM32F767xx) || defined(STM32F769xx)
 #define VBAT_DIV (4)
 #elif defined(STM32H743xx)
 #define VBAT_DIV (4)
-#elif defined(STM32L475xx) || defined(STM32L476xx)
+#elif defined(STM32L432xx) || defined(STM32L475xx) || \
+      defined(STM32L476xx) || defined(STM32L496xx)
 #define VBAT_DIV (3)
 #else
 #error Unsupported processor
@@ -132,7 +161,7 @@
 #define CORE_TEMP_AVG_SLOPE    (3)    /* (2.5mv/3.3v)*(2^ADC resoultion) */
 
 // scale and calibration values for VBAT and VREF
-#define ADC_SCALE (3.3f / 4095)
+#define ADC_SCALE (ADC_SCALE_V / ((1 << ADC_CAL_BITS) - 1))
 #define VREFIN_CAL ((uint16_t *)ADC_CAL_ADDRESS)
 
 typedef struct _pyb_obj_adc_t {
@@ -158,7 +187,7 @@ STATIC bool is_adcx_channel(int channel) {
 #if defined(STM32F411xE)
     // The HAL has an incorrect IS_ADC_CHANNEL macro for the F411 so we check for temp
     return IS_ADC_CHANNEL(channel) || channel == ADC_CHANNEL_TEMPSENSOR;
-#elif defined(STM32F4) || defined(STM32F7) || defined(STM32H7)
+#elif defined(STM32F0) || defined(STM32F4) || defined(STM32F7) || defined(STM32H7)
     return IS_ADC_CHANNEL(channel);
 #elif defined(STM32L4)
     ADC_HandleTypeDef handle;
@@ -173,7 +202,7 @@ STATIC void adc_wait_for_eoc_or_timeout(int32_t timeout) {
     uint32_t tickstart = HAL_GetTick();
 #if defined(STM32F4) || defined(STM32F7)
     while ((ADCx->SR & ADC_FLAG_EOC) != ADC_FLAG_EOC) {
-#elif defined(STM32H7) || defined(STM32L4)
+#elif defined(STM32F0) || defined(STM32H7) || defined(STM32L4)
     while (READ_BIT(ADCx->ISR, ADC_FLAG_EOC) != ADC_FLAG_EOC) {
 #else
     #error Unsupported processor
@@ -185,7 +214,7 @@ STATIC void adc_wait_for_eoc_or_timeout(int32_t timeout) {
 }
 
 STATIC void adcx_clock_enable(void) {
-#if defined(STM32F4) || defined(STM32F7)
+#if defined(STM32F0) || defined(STM32F4) || defined(STM32F7)
     ADCx_CLK_ENABLE();
 #elif defined(STM32H7)
     __HAL_RCC_ADC3_CLK_ENABLE();
@@ -204,12 +233,14 @@ STATIC void adcx_init_periph(ADC_HandleTypeDef *adch, uint32_t resolution) {
     adch->Init.Resolution            = resolution;
     adch->Init.ContinuousConvMode    = DISABLE;
     adch->Init.DiscontinuousConvMode = DISABLE;
+    #if !defined(STM32F0)
     adch->Init.NbrOfDiscConversion   = 0;
     adch->Init.NbrOfConversion       = 1;
+    #endif
     adch->Init.EOCSelection          = ADC_EOC_SINGLE_CONV;
     adch->Init.ExternalTrigConv      = ADC_SOFTWARE_START;
     adch->Init.ExternalTrigConvEdge  = ADC_EXTERNALTRIGCONVEDGE_NONE;
-    #if defined(STM32F4) || defined(STM32F7)
+    #if defined(STM32F0) || defined(STM32F4) || defined(STM32F7)
     adch->Init.ClockPrescaler        = ADC_CLOCK_SYNC_PCLK_DIV2;
     adch->Init.ScanConvMode          = DISABLE;
     adch->Init.DataAlign             = ADC_DATAALIGN_RIGHT;
@@ -235,11 +266,18 @@ STATIC void adcx_init_periph(ADC_HandleTypeDef *adch, uint32_t resolution) {
     #error Unsupported processor
     #endif
 
+    #if defined(STM32F0)
+    adch->Init.SamplingTimeCommon = ADC_SAMPLETIME_71CYCLES_5;
+    #endif
+
     HAL_ADC_Init(adch);
 
     #if defined(STM32H7)
     HAL_ADCEx_Calibration_Start(adch, ADC_CALIB_OFFSET, ADC_SINGLE_ENDED);
     #endif
+    #if defined(STM32L4)
+    HAL_ADCEx_Calibration_Start(adch, ADC_SINGLE_ENDED);
+    #endif
 }
 
 STATIC void adc_init_single(pyb_obj_adc_t *adc_obj) {
@@ -249,13 +287,13 @@ STATIC void adc_init_single(pyb_obj_adc_t *adc_obj) {
 
     if (ADC_FIRST_GPIO_CHANNEL <= adc_obj->channel && adc_obj->channel <= ADC_LAST_GPIO_CHANNEL) {
         // Channels 0-16 correspond to real pins. Configure the GPIO pin in ADC mode.
-        const pin_obj_t *pin = pin_adc1[adc_obj->channel];
+        const pin_obj_t *pin = pin_adc_table[adc_obj->channel];
         mp_hal_pin_config(pin, MP_HAL_PIN_MODE_ADC, MP_HAL_PIN_PULL_NONE, 0);
     }
 
     adcx_init_periph(&adc_obj->handle, ADC_RESOLUTION_12B);
 
-#if defined(STM32L4)
+#if defined(STM32L4) && defined(ADC_DUALMODE_REGSIMULT_INJECSIMULT)
     ADC_MultiModeTypeDef multimode;
     multimode.Mode = ADC_MODE_INDEPENDENT;
     if (HAL_ADCEx_MultiModeConfigChannel(&adc_obj->handle, &multimode) != HAL_OK)
@@ -270,22 +308,42 @@ STATIC void adc_config_channel(ADC_HandleTypeDef *adc_handle, uint32_t channel)
 
     sConfig.Channel = channel;
     sConfig.Rank = 1;
-#if defined(STM32F4) || defined(STM32F7)
+#if defined(STM32F0)
+    sConfig.SamplingTime = ADC_SAMPLETIME_71CYCLES_5;
+#elif defined(STM32F4) || defined(STM32F7)
     sConfig.SamplingTime = ADC_SAMPLETIME_15CYCLES;
 #elif defined(STM32H7)
-    sConfig.SamplingTime = ADC_SAMPLETIME_8CYCLES_5;
+    if (channel == ADC_CHANNEL_VREFINT
+        || channel == ADC_CHANNEL_TEMPSENSOR
+        || channel == ADC_CHANNEL_VBAT) {
+        sConfig.SamplingTime = ADC_SAMPLETIME_387CYCLES_5;
+    } else {
+        sConfig.SamplingTime = ADC_SAMPLETIME_8CYCLES_5;
+    }
     sConfig.SingleDiff = ADC_SINGLE_ENDED;
     sConfig.OffsetNumber = ADC_OFFSET_NONE;
     sConfig.OffsetRightShift = DISABLE;
     sConfig.OffsetSignedSaturation = DISABLE;
 #elif defined(STM32L4)
-    sConfig.SamplingTime = ADC_SAMPLETIME_12CYCLES_5;
+    if (channel == ADC_CHANNEL_VREFINT
+        || channel == ADC_CHANNEL_TEMPSENSOR
+        || channel == ADC_CHANNEL_VBAT) {
+        sConfig.SamplingTime = ADC_SAMPLETIME_247CYCLES_5;
+    } else {
+        sConfig.SamplingTime = ADC_SAMPLETIME_12CYCLES_5;
+    }
     sConfig.SingleDiff = ADC_SINGLE_ENDED;
     sConfig.OffsetNumber = ADC_OFFSET_NONE;
+    sConfig.Offset = 0;
 #else
     #error Unsupported processor
 #endif
-    sConfig.Offset = 0;
+
+    #if defined(STM32F0)
+    // On the STM32F0 we must select only one channel at a time to sample, so clear all
+    // channels before calling HAL_ADC_ConfigChannel, which will select the desired one.
+    adc_handle->Instance->CHSELR = 0;
+    #endif
 
     HAL_ADC_ConfigChannel(adc_handle, &sConfig);
 }
@@ -300,14 +358,27 @@ STATIC uint32_t adc_read_channel(ADC_HandleTypeDef *adcHandle) {
 
 STATIC uint32_t adc_config_and_read_channel(ADC_HandleTypeDef *adcHandle, uint32_t channel) {
     adc_config_channel(adcHandle, channel);
-    return adc_read_channel(adcHandle);
+    uint32_t raw_value = adc_read_channel(adcHandle);
+
+    #if defined(STM32F4) || defined(STM32F7)
+    // ST docs say that (at least on STM32F42x and STM32F43x), VBATE must
+    // be disabled when TSVREFE is enabled for TEMPSENSOR and VREFINT
+    // conversions to work.  VBATE is enabled by the above call to read
+    // the channel, and here we disable VBATE so a subsequent call for
+    // TEMPSENSOR or VREFINT works correctly.
+    if (channel == ADC_CHANNEL_VBAT) {
+        ADC->CCR &= ~ADC_CCR_VBATE;
+    }
+    #endif
+
+    return raw_value;
 }
 
 /******************************************************************************/
 /* MicroPython bindings : adc object (single channel)                         */
 
 STATIC void adc_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
-    pyb_obj_adc_t *self = self_in;
+    pyb_obj_adc_t *self = MP_OBJ_TO_PTR(self_in);
     mp_print_str(print, "<ADC on ");
     mp_obj_print_helper(print, self->pin_name, PRINT_STR);
     mp_printf(print, " channel=%u>", self->channel);
@@ -325,11 +396,11 @@ STATIC mp_obj_t adc_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_
 
     uint32_t channel;
 
-    if (MP_OBJ_IS_INT(pin_obj)) {
+    if (mp_obj_is_int(pin_obj)) {
         channel = adc_get_internal_channel(mp_obj_get_int(pin_obj));
     } else {
         const pin_obj_t *pin = pin_find(pin_obj);
-        if ((pin->adc_num & PIN_ADC1) == 0) {
+        if ((pin->adc_num & PIN_ADC_MASK) == 0) {
             // No ADC1 function on that pin
             nlr_raise(mp_obj_new_exception_msg_varg(&mp_type_ValueError, "pin %q does not have ADC capabilities", pin->name));
         }
@@ -343,7 +414,7 @@ STATIC mp_obj_t adc_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_
 
     if (ADC_FIRST_GPIO_CHANNEL <= channel && channel <= ADC_LAST_GPIO_CHANNEL) {
         // these channels correspond to physical GPIO ports so make sure they exist
-        if (pin_adc1[channel] == NULL) {
+        if (pin_adc_table[channel] == NULL) {
             nlr_raise(mp_obj_new_exception_msg_varg(&mp_type_ValueError,
                 "channel %d not available on this board", channel));
         }
@@ -356,14 +427,14 @@ STATIC mp_obj_t adc_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_
     o->channel = channel;
     adc_init_single(o);
 
-    return o;
+    return MP_OBJ_FROM_PTR(o);
 }
 
 /// \method read()
 /// Read the value on the analog pin and return it.  The returned value
 /// will be between 0 and 4095.
 STATIC mp_obj_t adc_read(mp_obj_t self_in) {
-    pyb_obj_adc_t *self = self_in;
+    pyb_obj_adc_t *self = MP_OBJ_TO_PTR(self_in);
     return mp_obj_new_int(adc_config_and_read_channel(&self->handle, self->channel));
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_1(adc_read_obj, adc_read);
@@ -403,7 +474,7 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_1(adc_read_obj, adc_read);
 ///
 /// This function does not allocate any memory.
 STATIC mp_obj_t adc_read_timed(mp_obj_t self_in, mp_obj_t buf_in, mp_obj_t freq_in) {
-    pyb_obj_adc_t *self = self_in;
+    pyb_obj_adc_t *self = MP_OBJ_TO_PTR(self_in);
 
     mp_buffer_info_t bufinfo;
     mp_get_buffer_raise(buf_in, &bufinfo, MP_BUFFER_WRITE);
@@ -442,7 +513,7 @@ STATIC mp_obj_t adc_read_timed(mp_obj_t self_in, mp_obj_t buf_in, mp_obj_t freq_
             // for subsequent samples we can just set the "start sample" bit
 #if defined(STM32F4) || defined(STM32F7)
             ADCx->CR2 |= (uint32_t)ADC_CR2_SWSTART;
-#elif defined(STM32H7) || defined(STM32L4)
+#elif defined(STM32F0) || defined(STM32H7) || defined(STM32L4)
             SET_BIT(ADCx->CR, ADC_CR_ADSTART);
 #else
             #error Unsupported processor
@@ -516,7 +587,7 @@ STATIC mp_obj_t adc_read_timed_multi(mp_obj_t adc_array_in, mp_obj_t buf_array_i
     tim = pyb_timer_get_handle(tim_in);
 
     // Start adc; this is slow so wait for it to start
-    pyb_obj_adc_t *adc0 = adc_array[0];
+    pyb_obj_adc_t *adc0 = MP_OBJ_TO_PTR(adc_array[0]);
     adc_config_channel(&adc0->handle, adc0->channel);
     HAL_ADC_Start(&adc0->handle);
     // Wait for sample to complete and discard
@@ -545,14 +616,14 @@ STATIC mp_obj_t adc_read_timed_multi(mp_obj_t adc_array_in, mp_obj_t buf_array_i
         __HAL_TIM_CLEAR_FLAG(tim, TIM_FLAG_UPDATE);
 
         for (size_t array_index = 0; array_index < nadcs; array_index++) {
-            pyb_obj_adc_t *adc = adc_array[array_index];
+            pyb_obj_adc_t *adc = MP_OBJ_TO_PTR(adc_array[array_index]);
             // configure the ADC channel
             adc_config_channel(&adc->handle, adc->channel);
             // for the first sample we need to turn the ADC on
             // ADC is started: set the "start sample" bit
             #if defined(STM32F4) || defined(STM32F7)
             ADCx->CR2 |= (uint32_t)ADC_CR2_SWSTART;
-            #elif defined(STM32H7) || defined(STM32L4)
+            #elif defined(STM32F0) || defined(STM32H7) || defined(STM32L4)
             SET_BIT(ADCx->CR, ADC_CR_ADSTART);
             #else
             #error Unsupported processor
@@ -572,7 +643,7 @@ STATIC mp_obj_t adc_read_timed_multi(mp_obj_t adc_array_in, mp_obj_t buf_array_i
     }
 
     // Turn the ADC off
-    adc0 = adc_array[0];
+    adc0 = MP_OBJ_TO_PTR(adc_array[0]);
     HAL_ADC_Stop(&adc0->handle);
 
     return mp_obj_new_bool(success);
@@ -613,6 +684,9 @@ void adc_init_all(pyb_adc_all_obj_t *adc_all, uint32_t resolution, uint32_t en_m
         case 8:  resolution = ADC_RESOLUTION_8B;  break;
         case 10: resolution = ADC_RESOLUTION_10B; break;
         case 12: resolution = ADC_RESOLUTION_12B; break;
+        #if defined(STM32H7)
+        case 16: resolution = ADC_RESOLUTION_16B; break;
+        #endif
         default:
             nlr_raise(mp_obj_new_exception_msg_varg(&mp_type_ValueError,
                 "resolution %d not supported", resolution));
@@ -623,7 +697,7 @@ void adc_init_all(pyb_adc_all_obj_t *adc_all, uint32_t resolution, uint32_t en_m
         if (en_mask & (1 << channel)) {
             // Channels 0-16 correspond to real pins. Configure the GPIO pin in
             // ADC mode.
-            const pin_obj_t *pin = pin_adc1[channel];
+            const pin_obj_t *pin = pin_adc_table[channel];
             if (pin) {
                 mp_hal_pin_config(pin, MP_HAL_PIN_MODE_ADC, MP_HAL_PIN_PULL_NONE, 0);
             }
@@ -642,17 +716,21 @@ int adc_get_resolution(ADC_HandleTypeDef *adcHandle) {
         #endif
         case ADC_RESOLUTION_8B:  return 8;
         case ADC_RESOLUTION_10B: return 10;
+        #if defined(STM32H7)
+        case ADC_RESOLUTION_16B: return 16;
+        #endif
     }
     return 12;
 }
 
+STATIC uint32_t adc_config_and_read_ref(ADC_HandleTypeDef *adcHandle, uint32_t channel) {
+    uint32_t raw_value = adc_config_and_read_channel(adcHandle, channel);
+    // Scale raw reading to the number of bits used by the calibration constants
+    return raw_value << (ADC_CAL_BITS - adc_get_resolution(adcHandle));
+}
+
 int adc_read_core_temp(ADC_HandleTypeDef *adcHandle) {
-    int32_t raw_value = adc_config_and_read_channel(adcHandle, ADC_CHANNEL_TEMPSENSOR);
-
-    // Note: constants assume 12-bit resolution, so we scale the raw value to
-    //       be 12-bits.
-    raw_value <<= (12 - adc_get_resolution(adcHandle));
-
+    int32_t raw_value = adc_config_and_read_ref(adcHandle, ADC_CHANNEL_TEMPSENSOR);
     return ((raw_value - CORE_TEMP_V25) / CORE_TEMP_AVG_SLOPE) + 25;
 }
 
@@ -661,40 +739,18 @@ int adc_read_core_temp(ADC_HandleTypeDef *adcHandle) {
 STATIC volatile float adc_refcor = 1.0f;
 
 float adc_read_core_temp_float(ADC_HandleTypeDef *adcHandle) {
-    int32_t raw_value = adc_config_and_read_channel(adcHandle, ADC_CHANNEL_TEMPSENSOR);
-
-    // constants assume 12-bit resolution so we scale the raw value to 12-bits
-    raw_value <<= (12 - adc_get_resolution(adcHandle));
-
+    int32_t raw_value = adc_config_and_read_ref(adcHandle, ADC_CHANNEL_TEMPSENSOR);
     float core_temp_avg_slope = (*ADC_CAL2 - *ADC_CAL1) / 80.0;
     return (((float)raw_value * adc_refcor - *ADC_CAL1) / core_temp_avg_slope) + 30.0f;
 }
 
 float adc_read_core_vbat(ADC_HandleTypeDef *adcHandle) {
-    uint32_t raw_value = adc_config_and_read_channel(adcHandle, ADC_CHANNEL_VBAT);
-
-    // Note: constants assume 12-bit resolution, so we scale the raw value to
-    //       be 12-bits.
-    raw_value <<= (12 - adc_get_resolution(adcHandle));
-
-    #if defined(STM32F4) || defined(STM32F7)
-    // ST docs say that (at least on STM32F42x and STM32F43x), VBATE must
-    // be disabled when TSVREFE is enabled for TEMPSENSOR and VREFINT
-    // conversions to work.  VBATE is enabled by the above call to read
-    // the channel, and here we disable VBATE so a subsequent call for
-    // TEMPSENSOR or VREFINT works correctly.
-    ADC->CCR &= ~ADC_CCR_VBATE;
-    #endif
-
+    uint32_t raw_value = adc_config_and_read_ref(adcHandle, ADC_CHANNEL_VBAT);
     return raw_value * VBAT_DIV * ADC_SCALE * adc_refcor;
 }
 
 float adc_read_core_vref(ADC_HandleTypeDef *adcHandle) {
-    uint32_t raw_value = adc_config_and_read_channel(adcHandle, ADC_CHANNEL_VREFINT);
-
-    // Note: constants assume 12-bit resolution, so we scale the raw value to
-    //       be 12-bits.
-    raw_value <<= (12 - adc_get_resolution(adcHandle));
+    uint32_t raw_value = adc_config_and_read_ref(adcHandle, ADC_CHANNEL_VREFINT);
 
     // update the reference correction factor
     adc_refcor = ((float)(*VREFIN_CAL)) / ((float)raw_value);
@@ -720,11 +776,11 @@ STATIC mp_obj_t adc_all_make_new(const mp_obj_type_t *type, size_t n_args, size_
     }
     adc_init_all(o, res, en_mask);
 
-    return o;
+    return MP_OBJ_FROM_PTR(o);
 }
 
 STATIC mp_obj_t adc_all_read_channel(mp_obj_t self_in, mp_obj_t channel) {
-    pyb_adc_all_obj_t *self = self_in;
+    pyb_adc_all_obj_t *self = MP_OBJ_TO_PTR(self_in);
     uint32_t chan = adc_get_internal_channel(mp_obj_get_int(channel));
     uint32_t data = adc_config_and_read_channel(&self->handle, chan);
     return mp_obj_new_int(data);
@@ -732,7 +788,7 @@ STATIC mp_obj_t adc_all_read_channel(mp_obj_t self_in, mp_obj_t channel) {
 STATIC MP_DEFINE_CONST_FUN_OBJ_2(adc_all_read_channel_obj, adc_all_read_channel);
 
 STATIC mp_obj_t adc_all_read_core_temp(mp_obj_t self_in) {
-    pyb_adc_all_obj_t *self = self_in;
+    pyb_adc_all_obj_t *self = MP_OBJ_TO_PTR(self_in);
     #if MICROPY_PY_BUILTINS_FLOAT
     float data = adc_read_core_temp_float(&self->handle);
     return mp_obj_new_float(data);
@@ -745,23 +801,23 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_1(adc_all_read_core_temp_obj, adc_all_read_core_t
 
 #if MICROPY_PY_BUILTINS_FLOAT
 STATIC mp_obj_t adc_all_read_core_vbat(mp_obj_t self_in) {
-    pyb_adc_all_obj_t *self = self_in;
+    pyb_adc_all_obj_t *self = MP_OBJ_TO_PTR(self_in);
     float data = adc_read_core_vbat(&self->handle);
     return mp_obj_new_float(data);
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_1(adc_all_read_core_vbat_obj, adc_all_read_core_vbat);
 
 STATIC mp_obj_t adc_all_read_core_vref(mp_obj_t self_in) {
-    pyb_adc_all_obj_t *self = self_in;
+    pyb_adc_all_obj_t *self = MP_OBJ_TO_PTR(self_in);
     float data  = adc_read_core_vref(&self->handle);
     return mp_obj_new_float(data);
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_1(adc_all_read_core_vref_obj, adc_all_read_core_vref);
 
 STATIC mp_obj_t adc_all_read_vref(mp_obj_t self_in) {
-    pyb_adc_all_obj_t *self = self_in;
+    pyb_adc_all_obj_t *self = MP_OBJ_TO_PTR(self_in);
     adc_read_core_vref(&self->handle);
-    return mp_obj_new_float(3.3 * adc_refcor);
+    return mp_obj_new_float(ADC_SCALE_V * adc_refcor);
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_1(adc_all_read_vref_obj, adc_all_read_vref);
 #endif
