@@ -114,7 +114,7 @@ STATIC bool can_init(pyb_can_obj_t *can_obj) {
             sce_irq = CAN1_SCE_IRQn;
             pins[0] = MICROPY_HW_CAN1_TX;
             pins[1] = MICROPY_HW_CAN1_RX;
-            __CAN1_CLK_ENABLE();
+            __HAL_RCC_CAN1_CLK_ENABLE();
             break;
         #endif
 
@@ -124,8 +124,18 @@ STATIC bool can_init(pyb_can_obj_t *can_obj) {
             sce_irq = CAN2_SCE_IRQn;
             pins[0] = MICROPY_HW_CAN2_TX;
             pins[1] = MICROPY_HW_CAN2_RX;
-            __CAN1_CLK_ENABLE(); // CAN2 is a "slave" and needs CAN1 enabled as well
-            __CAN2_CLK_ENABLE();
+            __HAL_RCC_CAN1_CLK_ENABLE(); // CAN2 is a "slave" and needs CAN1 enabled as well
+            __HAL_RCC_CAN2_CLK_ENABLE();
+            break;
+        #endif
+
+        #if defined(MICROPY_HW_CAN3_TX)
+        case PYB_CAN_3:
+            CANx = CAN3;
+            sce_irq = CAN3_SCE_IRQn;
+            pins[0] = MICROPY_HW_CAN3_TX;
+            pins[1] = MICROPY_HW_CAN3_RX;
+            __HAL_RCC_CAN3_CLK_ENABLE(); // CAN3 is a "master" and doesn't need CAN1 enabled as well
             break;
         #endif
 
@@ -169,7 +179,7 @@ void can_deinit(void) {
     for (int i = 0; i < MP_ARRAY_SIZE(MP_STATE_PORT(pyb_can_obj_all)); i++) {
         pyb_can_obj_t *can_obj = MP_STATE_PORT(pyb_can_obj_all)[i];
         if (can_obj != NULL) {
-            pyb_can_deinit(can_obj);
+            pyb_can_deinit(MP_OBJ_FROM_PTR(can_obj));
         }
     }
 }
@@ -330,7 +340,7 @@ STATIC HAL_StatusTypeDef CAN_Transmit(CAN_HandleTypeDef *hcan, uint32_t Timeout)
 // MicroPython bindings
 
 STATIC void pyb_can_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
-    pyb_can_obj_t *self = self_in;
+    pyb_can_obj_t *self = MP_OBJ_TO_PTR(self_in);
     if (!self->is_enabled) {
         mp_printf(print, "CAN(%u)", self->can_id);
     } else {
@@ -409,7 +419,7 @@ STATIC mp_obj_t pyb_can_make_new(const mp_obj_type_t *type, size_t n_args, size_
 
     // work out port
     mp_uint_t can_idx;
-    if (MP_OBJ_IS_STR(args[0])) {
+    if (mp_obj_is_str(args[0])) {
         const char *port = mp_obj_str_get_str(args[0]);
         if (0) {
         #ifdef MICROPY_HW_CAN1_NAME
@@ -420,6 +430,10 @@ STATIC mp_obj_t pyb_can_make_new(const mp_obj_type_t *type, size_t n_args, size_
         } else if (strcmp(port, MICROPY_HW_CAN2_NAME) == 0) {
             can_idx = PYB_CAN_2;
         #endif
+        #ifdef MICROPY_HW_CAN3_NAME
+        } else if (strcmp(port, MICROPY_HW_CAN3_NAME) == 0) {
+            can_idx = PYB_CAN_3;
+        #endif
         } else {
             nlr_raise(mp_obj_new_exception_msg_varg(&mp_type_ValueError, "CAN(%s) doesn't exist", port));
         }
@@ -445,7 +459,7 @@ STATIC mp_obj_t pyb_can_make_new(const mp_obj_type_t *type, size_t n_args, size_
         if (self->is_enabled) {
             // The caller is requesting a reconfiguration of the hardware
             // this can only be done if the hardware is in init mode
-            pyb_can_deinit(self);
+            pyb_can_deinit(MP_OBJ_FROM_PTR(self));
         }
 
         self->rxcallback0 = mp_const_none;
@@ -461,35 +475,44 @@ STATIC mp_obj_t pyb_can_make_new(const mp_obj_type_t *type, size_t n_args, size_
         }
     }
 
-    return self;
+    return MP_OBJ_FROM_PTR(self);
 }
 
 STATIC mp_obj_t pyb_can_init(size_t n_args, const mp_obj_t *args, mp_map_t *kw_args) {
-    return pyb_can_init_helper(args[0], n_args - 1, args + 1, kw_args);
+    return pyb_can_init_helper(MP_OBJ_TO_PTR(args[0]), n_args - 1, args + 1, kw_args);
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_KW(pyb_can_init_obj, 1, pyb_can_init);
 
 /// \method deinit()
 /// Turn off the CAN bus.
 STATIC mp_obj_t pyb_can_deinit(mp_obj_t self_in) {
-    pyb_can_obj_t *self = self_in;
+    pyb_can_obj_t *self = MP_OBJ_TO_PTR(self_in);
     self->is_enabled = false;
     HAL_CAN_DeInit(&self->can);
     if (self->can.Instance == CAN1) {
         HAL_NVIC_DisableIRQ(CAN1_RX0_IRQn);
         HAL_NVIC_DisableIRQ(CAN1_RX1_IRQn);
         HAL_NVIC_DisableIRQ(CAN1_SCE_IRQn);
-        __CAN1_FORCE_RESET();
-        __CAN1_RELEASE_RESET();
-        __CAN1_CLK_DISABLE();
+        __HAL_RCC_CAN1_FORCE_RESET();
+        __HAL_RCC_CAN1_RELEASE_RESET();
+        __HAL_RCC_CAN1_CLK_DISABLE();
     #if defined(CAN2)
     } else if (self->can.Instance == CAN2) {
         HAL_NVIC_DisableIRQ(CAN2_RX0_IRQn);
         HAL_NVIC_DisableIRQ(CAN2_RX1_IRQn);
         HAL_NVIC_DisableIRQ(CAN2_SCE_IRQn);
-        __CAN2_FORCE_RESET();
-        __CAN2_RELEASE_RESET();
-        __CAN2_CLK_DISABLE();
+        __HAL_RCC_CAN2_FORCE_RESET();
+        __HAL_RCC_CAN2_RELEASE_RESET();
+        __HAL_RCC_CAN2_CLK_DISABLE();
+    #endif
+    #if defined(CAN3)
+    } else if (self->can.Instance == CAN3) {
+        HAL_NVIC_DisableIRQ(CAN3_RX0_IRQn);
+        HAL_NVIC_DisableIRQ(CAN3_RX1_IRQn);
+        HAL_NVIC_DisableIRQ(CAN3_SCE_IRQn);
+        __HAL_RCC_CAN3_FORCE_RESET();
+        __HAL_RCC_CAN3_RELEASE_RESET();
+        __HAL_RCC_CAN3_CLK_DISABLE();
     #endif
     }
     return mp_const_none;
@@ -540,7 +563,7 @@ STATIC mp_obj_t pyb_can_info(size_t n_args, const mp_obj_t *args) {
     if (n_args == 1) {
         list = MP_OBJ_TO_PTR(mp_obj_new_list(8, NULL));
     } else {
-        if (!MP_OBJ_IS_TYPE(args[1], &mp_type_list)) {
+        if (!mp_obj_is_type(args[1], &mp_type_list)) {
             mp_raise_TypeError(NULL);
         }
         list = MP_OBJ_TO_PTR(args[1]);
@@ -566,7 +589,7 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(pyb_can_info_obj, 1, 2, pyb_can_info)
 /// \method any(fifo)
 /// Return `True` if any message waiting on the FIFO, else `False`.
 STATIC mp_obj_t pyb_can_any(mp_obj_t self_in, mp_obj_t fifo_in) {
-    pyb_can_obj_t *self = self_in;
+    pyb_can_obj_t *self = MP_OBJ_TO_PTR(self_in);
     mp_int_t fifo = mp_obj_get_int(fifo_in);
     if (fifo == 0) {
         if (__HAL_CAN_MSG_PENDING(&self->can, CAN_FIFO0) != 0) {
@@ -599,7 +622,7 @@ STATIC mp_obj_t pyb_can_send(size_t n_args, const mp_obj_t *pos_args, mp_map_t *
     };
 
     // parse args
-    pyb_can_obj_t *self = pos_args[0];
+    pyb_can_obj_t *self = MP_OBJ_TO_PTR(pos_args[0]);
     mp_arg_val_t args[MP_ARRAY_SIZE(allowed_args)];
     mp_arg_parse_all(n_args - 1, pos_args + 1, kw_args, MP_ARRAY_SIZE(allowed_args), allowed_args, args);
 
@@ -655,12 +678,12 @@ STATIC mp_obj_t pyb_can_recv(size_t n_args, const mp_obj_t *pos_args, mp_map_t *
     enum { ARG_fifo, ARG_list, ARG_timeout };
     static const mp_arg_t allowed_args[] = {
         { MP_QSTR_fifo,    MP_ARG_REQUIRED | MP_ARG_INT, {.u_int = 0} },
-        { MP_QSTR_list,    MP_ARG_OBJ, {.u_obj = mp_const_none} },
+        { MP_QSTR_list,    MP_ARG_OBJ, {.u_rom_obj = MP_ROM_PTR(&mp_const_none_obj)} },
         { MP_QSTR_timeout, MP_ARG_KW_ONLY | MP_ARG_INT, {.u_int = 5000} },
     };
 
     // parse args
-    pyb_can_obj_t *self = pos_args[0];
+    pyb_can_obj_t *self = MP_OBJ_TO_PTR(pos_args[0]);
     mp_arg_val_t args[MP_ARRAY_SIZE(allowed_args)];
     mp_arg_parse_all(n_args - 1, pos_args + 1, kw_args, MP_ARRAY_SIZE(allowed_args), allowed_args, args);
 
@@ -709,7 +732,7 @@ STATIC mp_obj_t pyb_can_recv(size_t n_args, const mp_obj_t *pos_args, mp_map_t *
         items[3] = mp_obj_new_bytes(&rx_msg.Data[0], rx_msg.DLC);
     } else {
         // User should provide a list of length at least 4 to hold the values
-        if (!MP_OBJ_IS_TYPE(ret_obj, &mp_type_list)) {
+        if (!mp_obj_is_type(ret_obj, &mp_type_list)) {
             mp_raise_TypeError(NULL);
         }
         mp_obj_list_t *list = MP_OBJ_TO_PTR(ret_obj);
@@ -719,12 +742,12 @@ STATIC mp_obj_t pyb_can_recv(size_t n_args, const mp_obj_t *pos_args, mp_map_t *
         items = list->items;
         // Fourth element must be a memoryview which we assume points to a
         // byte-like array which is large enough, and then we resize it inplace
-        if (!MP_OBJ_IS_TYPE(items[3], &mp_type_memoryview)) {
+        if (!mp_obj_is_type(items[3], &mp_type_memoryview)) {
             mp_raise_TypeError(NULL);
         }
         mp_obj_array_t *mv = MP_OBJ_TO_PTR(items[3]);
-        if (!(mv->typecode == (0x80 | BYTEARRAY_TYPECODE)
-            || (mv->typecode | 0x20) == (0x80 | 'b'))) {
+        if (!(mv->typecode == (MP_OBJ_ARRAY_TYPECODE_FLAG_RW | BYTEARRAY_TYPECODE)
+            || (mv->typecode | 0x20) == (MP_OBJ_ARRAY_TYPECODE_FLAG_RW | 'b'))) {
             mp_raise_ValueError(NULL);
         }
         mv->len = rx_msg.DLC;
@@ -765,10 +788,10 @@ STATIC mp_obj_t pyb_can_initfilterbanks(mp_obj_t self, mp_obj_t bank_in) {
     return mp_const_none;
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_2(pyb_can_initfilterbanks_fun_obj, pyb_can_initfilterbanks);
-STATIC MP_DEFINE_CONST_CLASSMETHOD_OBJ(pyb_can_initfilterbanks_obj, (const mp_obj_t)&pyb_can_initfilterbanks_fun_obj);
+STATIC MP_DEFINE_CONST_CLASSMETHOD_OBJ(pyb_can_initfilterbanks_obj, MP_ROM_PTR(&pyb_can_initfilterbanks_fun_obj));
 
 STATIC mp_obj_t pyb_can_clearfilter(mp_obj_t self_in, mp_obj_t bank_in) {
-    pyb_can_obj_t *self = self_in;
+    pyb_can_obj_t *self = MP_OBJ_TO_PTR(self_in);
     mp_int_t f = mp_obj_get_int(bank_in);
     if (self->can_id == 2) {
         f += can2_start_bank;
@@ -792,7 +815,7 @@ STATIC mp_obj_t pyb_can_setfilter(size_t n_args, const mp_obj_t *pos_args, mp_ma
     };
 
     // parse args
-    pyb_can_obj_t *self = pos_args[0];
+    pyb_can_obj_t *self = MP_OBJ_TO_PTR(pos_args[0]);
     mp_arg_val_t args[MP_ARRAY_SIZE(allowed_args)];
     mp_arg_parse_all(n_args - 1, pos_args + 1, kw_args, MP_ARRAY_SIZE(allowed_args), allowed_args, args);
 
@@ -890,11 +913,15 @@ STATIC mp_obj_t pyb_can_setfilter(size_t n_args, const mp_obj_t *pos_args, mp_ma
         if (filter.FilterNumber >= can2_start_bank) {
             goto error;
         }
-    } else {
+    } else if (self->can_id == 2) {
         filter.FilterNumber = filter.FilterNumber + can2_start_bank;
         if (filter.FilterNumber > 27) {
             goto error;
         }
+    } else {
+        if (filter.FilterNumber > 13) { // CAN3 is independant and has its own 14 filters.
+            goto error;
+        }
     }
     filter.FilterActivation = ENABLE;
     filter.BankNumber = can2_start_bank;
@@ -908,7 +935,7 @@ error:
 STATIC MP_DEFINE_CONST_FUN_OBJ_KW(pyb_can_setfilter_obj, 1, pyb_can_setfilter);
 
 STATIC mp_obj_t pyb_can_rxcallback(mp_obj_t self_in, mp_obj_t fifo_in, mp_obj_t callback_in) {
-    pyb_can_obj_t *self = self_in;
+    pyb_can_obj_t *self = MP_OBJ_TO_PTR(self_in);
     mp_int_t fifo = mp_obj_get_int(fifo_in);
     mp_obj_t *callback;
 
@@ -930,9 +957,13 @@ STATIC mp_obj_t pyb_can_rxcallback(mp_obj_t self_in, mp_obj_t fifo_in, mp_obj_t
         if (self->can_id == PYB_CAN_1) {
             irq = (fifo == 0) ? CAN1_RX0_IRQn : CAN1_RX1_IRQn;
         #if defined(CAN2)
-        } else {
+        } else if (self->can_id == PYB_CAN_2) {
             irq = (fifo == 0) ? CAN2_RX0_IRQn : CAN2_RX1_IRQn;
         #endif
+        #if defined(CAN3)
+        } else {
+            irq = (fifo == 0) ? CAN3_RX0_IRQn : CAN3_RX1_IRQn;
+        #endif
         }
         NVIC_SetPriority(irq, IRQ_PRI_CAN);
         HAL_NVIC_EnableIRQ(irq);
@@ -981,11 +1012,11 @@ STATIC const mp_rom_map_elem_t pyb_can_locals_dict_table[] = {
 
 STATIC MP_DEFINE_CONST_DICT(pyb_can_locals_dict, pyb_can_locals_dict_table);
 
-mp_uint_t can_ioctl(mp_obj_t self_in, mp_uint_t request, mp_uint_t arg, int *errcode) {
-    pyb_can_obj_t *self = self_in;
+mp_uint_t can_ioctl(mp_obj_t self_in, mp_uint_t request, uintptr_t arg, int *errcode) {
+    pyb_can_obj_t *self = MP_OBJ_TO_PTR(self_in);
     mp_uint_t ret;
     if (request == MP_STREAM_POLL) {
-        mp_uint_t flags = arg;
+        uintptr_t flags = arg;
         ret = 0;
         if ((flags & MP_STREAM_POLL_RD)
             && ((__HAL_CAN_MSG_PENDING(&self->can, CAN_FIFO0) != 0)
@@ -1046,13 +1077,13 @@ void can_rx_irq_handler(uint can_id, uint fifo_id) {
         gc_lock();
         nlr_buf_t nlr;
         if (nlr_push(&nlr) == 0) {
-            mp_call_function_2(callback, self, irq_reason);
+            mp_call_function_2(callback, MP_OBJ_FROM_PTR(self), irq_reason);
             nlr_pop();
         } else {
             // Uncaught exception; disable the callback so it doesn't run again.
-            pyb_can_rxcallback(self, MP_OBJ_NEW_SMALL_INT(fifo_id), mp_const_none);
+            pyb_can_rxcallback(MP_OBJ_FROM_PTR(self), MP_OBJ_NEW_SMALL_INT(fifo_id), mp_const_none);
             printf("uncaught exception in CAN(%u) rx interrupt handler\n", self->can_id);
-            mp_obj_print_exception(&mp_plat_print, (mp_obj_t)nlr.ret_val);
+            mp_obj_print_exception(&mp_plat_print, MP_OBJ_FROM_PTR(nlr.ret_val));
         }
         gc_unlock();
         mp_sched_unlock();
@@ -1087,7 +1118,7 @@ const mp_obj_type_t pyb_can_type = {
     .print = pyb_can_print,
     .make_new = pyb_can_make_new,
     .protocol = &can_stream_p,
-    .locals_dict = (mp_obj_t)&pyb_can_locals_dict,
+    .locals_dict = (mp_obj_dict_t*)&pyb_can_locals_dict,
 };
 
 #endif // MICROPY_HW_ENABLE_CAN
