@@ -93,18 +93,24 @@
 // The L4 MCU supports 40 Events/IRQs lines of the type configurable and direct.
 // Here we only support configurable line types.  Details, see page 330 of RM0351, Rev 1.
 // The USB_FS_WAKUP event is a direct type and there is no support for it.
-#define EXTI_Mode_Interrupt offsetof(EXTI_TypeDef, IMR1)
-#define EXTI_Mode_Event     offsetof(EXTI_TypeDef, EMR1)
+#define EXTI_Mode_Interrupt   offsetof(EXTI_TypeDef, IMR1)
+#define EXTI_Mode_Event       offsetof(EXTI_TypeDef, EMR1)
+#define EXTI_Trigger_Rising   offsetof(EXTI_TypeDef, RTSR1)
+#define EXTI_Trigger_Falling  offsetof(EXTI_TypeDef, FTSR1)
 #define EXTI_RTSR EXTI->RTSR1
 #define EXTI_FTSR EXTI->FTSR1
 #elif defined(STM32H7)
-#define EXTI_Mode_Interrupt offsetof(EXTI_Core_TypeDef, IMR1)
-#define EXTI_Mode_Event     offsetof(EXTI_Core_TypeDef, EMR1)
+#define EXTI_Mode_Interrupt   offsetof(EXTI_Core_TypeDef, IMR1)
+#define EXTI_Mode_Event       offsetof(EXTI_Core_TypeDef, EMR1)
+#define EXTI_Trigger_Rising   offsetof(EXTI_Core_TypeDef, RTSR1)
+#define EXTI_Trigger_Falling  offsetof(EXTI_Core_TypeDef, FTSR1)
 #define EXTI_RTSR EXTI->RTSR1
 #define EXTI_FTSR EXTI->FTSR1
 #else
-#define EXTI_Mode_Interrupt offsetof(EXTI_TypeDef, IMR)
-#define EXTI_Mode_Event     offsetof(EXTI_TypeDef, EMR)
+#define EXTI_Mode_Interrupt   offsetof(EXTI_TypeDef, IMR)
+#define EXTI_Mode_Event       offsetof(EXTI_TypeDef, EMR)
+#define EXTI_Trigger_Rising   offsetof(EXTI_TypeDef, RTSR)
+#define EXTI_Trigger_Falling  offsetof(EXTI_TypeDef, FTSR)
 #define EXTI_RTSR EXTI->RTSR
 #define EXTI_FTSR EXTI->FTSR
 #endif
@@ -133,6 +139,19 @@ STATIC mp_obj_t pyb_extint_callback_arg[EXTI_NUM_VECTORS];
 #endif
 
 STATIC const uint8_t nvic_irq_channel[EXTI_NUM_VECTORS] = {
+    #if defined(STM32F0)
+    EXTI0_1_IRQn,  EXTI0_1_IRQn,  EXTI2_3_IRQn,  EXTI2_3_IRQn,
+    EXTI4_15_IRQn, EXTI4_15_IRQn, EXTI4_15_IRQn, EXTI4_15_IRQn,
+    EXTI4_15_IRQn, EXTI4_15_IRQn, EXTI4_15_IRQn, EXTI4_15_IRQn,
+    EXTI4_15_IRQn, EXTI4_15_IRQn, EXTI4_15_IRQn, EXTI4_15_IRQn,
+    PVD_VDDIO2_IRQn,
+    RTC_IRQn,
+    0, // internal USB wakeup event
+    RTC_IRQn,
+    RTC_IRQn,
+    ADC1_COMP_IRQn,
+    ADC1_COMP_IRQn,
+    #else
     EXTI0_IRQn,     EXTI1_IRQn,     EXTI2_IRQn,     EXTI3_IRQn,     EXTI4_IRQn,
     EXTI9_5_IRQn,   EXTI9_5_IRQn,   EXTI9_5_IRQn,   EXTI9_5_IRQn,   EXTI9_5_IRQn,
     EXTI15_10_IRQn, EXTI15_10_IRQn, EXTI15_10_IRQn, EXTI15_10_IRQn, EXTI15_10_IRQn,
@@ -142,12 +161,18 @@ STATIC const uint8_t nvic_irq_channel[EXTI_NUM_VECTORS] = {
     #else
     PVD_IRQn,
     #endif
+    #if defined(STM32L4)
+    OTG_FS_WKUP_IRQn,
+    RTC_Alarm_IRQn,
+    #else
     RTC_Alarm_IRQn,
     OTG_FS_WKUP_IRQn,
+    #endif
     ETH_WKUP_IRQn,
     OTG_HS_WKUP_IRQn,
     TAMP_STAMP_IRQn,
     RTC_WKUP_IRQn,
+    #endif
 };
 
 // Set override_callback_obj to true if you want to unconditionally set the
@@ -156,7 +181,7 @@ uint extint_register(mp_obj_t pin_obj, uint32_t mode, uint32_t pull, mp_obj_t ca
     const pin_obj_t *pin = NULL;
     uint v_line;
 
-    if (MP_OBJ_IS_INT(pin_obj)) {
+    if (mp_obj_is_int(pin_obj)) {
         // If an integer is passed in, then use it to identify lines 16 thru 22
         // We expect lines 0 thru 15 to be passed in as a pin, so that we can
         // get both the port number and line number.
@@ -203,15 +228,21 @@ uint extint_register(mp_obj_t pin_obj, uint32_t mode, uint32_t pull, mp_obj_t ca
         pyb_extint_hard_irq[v_line] = true;
         pyb_extint_callback_arg[v_line] = MP_OBJ_NEW_SMALL_INT(v_line);
 
-        mp_hal_gpio_clock_enable(pin->gpio);
-        GPIO_InitTypeDef exti;
-        exti.Pin = pin->pin_mask;
-        exti.Mode = mode;
-        exti.Pull = pull;
-        exti.Speed = GPIO_SPEED_FREQ_HIGH;
-        HAL_GPIO_Init(pin->gpio, &exti);
+        if (pin == NULL) {
+            // pin will be NULL for non GPIO EXTI lines
+            extint_trigger_mode(v_line, mode);
+            extint_enable(v_line);
+        } else {
+            mp_hal_gpio_clock_enable(pin->gpio);
+            GPIO_InitTypeDef exti;
+            exti.Pin = pin->pin_mask;
+            exti.Mode = mode;
+            exti.Pull = pull;
+            exti.Speed = GPIO_SPEED_FREQ_HIGH;
+            HAL_GPIO_Init(pin->gpio, &exti);
 
-        // Calling HAL_GPIO_Init does an implicit extint_enable
+            // Calling HAL_GPIO_Init does an implicit extint_enable
+        }
 
         /* Enable and set NVIC Interrupt to the lowest priority */
         NVIC_SetPriority(IRQn_NONNEG(nvic_irq_channel[v_line]), IRQ_PRI_EXTINT);
@@ -227,11 +258,11 @@ void extint_register_pin(const pin_obj_t *pin, uint32_t mode, bool hard_irq, mp_
     // Check if the ExtInt line is already in use by another Pin/ExtInt
     mp_obj_t *cb = &MP_STATE_PORT(pyb_extint_callback)[line];
     if (*cb != mp_const_none && MP_OBJ_FROM_PTR(pin) != pyb_extint_callback_arg[line]) {
-        if (MP_OBJ_IS_SMALL_INT(pyb_extint_callback_arg[line])) {
+        if (mp_obj_is_small_int(pyb_extint_callback_arg[line])) {
             nlr_raise(mp_obj_new_exception_msg_varg(&mp_type_OSError,
                 "ExtInt vector %d is already in use", line));
         } else {
-            const pin_obj_t *other_pin = (const pin_obj_t*)pyb_extint_callback_arg[line];
+            const pin_obj_t *other_pin = MP_OBJ_TO_PTR(pyb_extint_callback_arg[line]);
             nlr_raise(mp_obj_new_exception_msg_varg(&mp_type_OSError,
                 "IRQ resource already taken by Pin('%q')", other_pin->name));
         }
@@ -255,19 +286,7 @@ void extint_register_pin(const pin_obj_t *pin, uint32_t mode, bool hard_irq, mp_
             (SYSCFG->EXTICR[line >> 2] & ~(0x0f << (4 * (line & 0x03))))
             | ((uint32_t)(GPIO_GET_INDEX(pin->gpio)) << (4 * (line & 0x03)));
 
-        // Enable or disable the rising detector
-        if ((mode & GPIO_MODE_IT_RISING) == GPIO_MODE_IT_RISING) {
-            EXTI_RTSR |= 1 << line;
-        } else {
-            EXTI_RTSR &= ~(1 << line);
-        }
-
-        // Enable or disable the falling detector
-        if ((mode & GPIO_MODE_IT_FALLING) == GPIO_MODE_IT_FALLING) {
-            EXTI_FTSR |= 1 << line;
-        } else {
-            EXTI_FTSR &= ~(1 << line);
-        }
+        extint_trigger_mode(line, mode);
 
         // Configure the NVIC
         NVIC_SetPriority(IRQn_NONNEG(nvic_irq_channel[line]), IRQ_PRI_EXTINT);
@@ -282,7 +301,7 @@ void extint_enable(uint line) {
     if (line >= EXTI_NUM_VECTORS) {
         return;
     }
-    #if defined(STM32F7) || defined(STM32H7)
+    #if defined(STM32F0) || defined(STM32F7) || defined(STM32H7)
     // The Cortex-M7 doesn't have bitband support.
     mp_uint_t irq_state = disable_irq();
     if (pyb_extint_mode[line] == EXTI_Mode_Interrupt) {
@@ -312,7 +331,7 @@ void extint_disable(uint line) {
         return;
     }
 
-    #if defined(STM32F7) || defined(STM32H7)
+    #if defined(STM32F0) || defined(STM32F7) || defined(STM32H7)
     // The Cortex-M7 doesn't have bitband support.
     mp_uint_t irq_state = disable_irq();
     #if defined(STM32H7)
@@ -346,10 +365,39 @@ void extint_swint(uint line) {
 #endif
 }
 
+void extint_trigger_mode(uint line, uint32_t mode) {
+    if (line >= EXTI_NUM_VECTORS) {
+        return;
+    }
+    #if defined(STM32F0) || defined(STM32F7) || defined(STM32H7)
+    // The Cortex-M7 doesn't have bitband support.
+    mp_uint_t irq_state = disable_irq();
+    // Enable or disable the rising detector
+    if ((mode & GPIO_MODE_IT_RISING) == GPIO_MODE_IT_RISING) {
+        EXTI_RTSR |= (1 << line);
+    } else {
+        EXTI_RTSR &= ~(1 << line);
+    }
+    // Enable or disable the falling detector
+    if ((mode & GPIO_MODE_IT_FALLING) == GPIO_MODE_IT_FALLING) {
+        EXTI_FTSR |= 1 << line;
+    } else {
+        EXTI_FTSR &= ~(1 << line);
+    }
+    enable_irq(irq_state);
+    #else
+    // Since manipulating FTSR/RTSR is a read-modify-write, and we want this to
+    // be atomic, we use the bit-band area to just affect the bit we're
+    // interested in.
+    EXTI_MODE_BB(EXTI_Trigger_Rising, line) = (mode & GPIO_MODE_IT_RISING) == GPIO_MODE_IT_RISING;
+    EXTI_MODE_BB(EXTI_Trigger_Falling, line) = (mode & GPIO_MODE_IT_FALLING) == GPIO_MODE_IT_FALLING;
+    #endif
+}
+
 /// \method line()
 /// Return the line number that the pin is mapped to.
 STATIC mp_obj_t extint_obj_line(mp_obj_t self_in) {
-    extint_obj_t *self = self_in;
+    extint_obj_t *self = MP_OBJ_TO_PTR(self_in);
     return MP_OBJ_NEW_SMALL_INT(self->line);
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_1(extint_obj_line_obj, extint_obj_line);
@@ -357,7 +405,7 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_1(extint_obj_line_obj, extint_obj_line);
 /// \method enable()
 /// Enable a disabled interrupt.
 STATIC mp_obj_t extint_obj_enable(mp_obj_t self_in) {
-    extint_obj_t *self = self_in;
+    extint_obj_t *self = MP_OBJ_TO_PTR(self_in);
     extint_enable(self->line);
     return mp_const_none;
 }
@@ -367,7 +415,7 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_1(extint_obj_enable_obj, extint_obj_enable);
 /// Disable the interrupt associated with the ExtInt object.
 /// This could be useful for debouncing.
 STATIC mp_obj_t extint_obj_disable(mp_obj_t self_in) {
-    extint_obj_t *self = self_in;
+    extint_obj_t *self = MP_OBJ_TO_PTR(self_in);
     extint_disable(self->line);
     return mp_const_none;
 }
@@ -376,7 +424,7 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_1(extint_obj_disable_obj, extint_obj_disable);
 /// \method swint()
 /// Trigger the callback from software.
 STATIC mp_obj_t extint_obj_swint(mp_obj_t self_in) {
-    extint_obj_t *self = self_in;
+    extint_obj_t *self = MP_OBJ_TO_PTR(self_in);
     extint_swint(self->line);
     return mp_const_none;
 }
@@ -429,7 +477,7 @@ STATIC mp_obj_t extint_regs(void) {
     return mp_const_none;
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_0(extint_regs_fun_obj, extint_regs);
-STATIC MP_DEFINE_CONST_STATICMETHOD_OBJ(extint_regs_obj, (mp_obj_t)&extint_regs_fun_obj);
+STATIC MP_DEFINE_CONST_STATICMETHOD_OBJ(extint_regs_obj, MP_ROM_PTR(&extint_regs_fun_obj));
 
 /// \classmethod \constructor(pin, mode, pull, callback)
 /// Create an ExtInt object:
@@ -465,11 +513,11 @@ STATIC mp_obj_t extint_make_new(const mp_obj_type_t *type, size_t n_args, size_t
     self->base.type = type;
     self->line = extint_register(vals[0].u_obj, vals[1].u_int, vals[2].u_int, vals[3].u_obj, false);
 
-    return self;
+    return MP_OBJ_FROM_PTR(self);
 }
 
 STATIC void extint_obj_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
-    extint_obj_t *self = self_in;
+    extint_obj_t *self = MP_OBJ_TO_PTR(self_in);
     mp_printf(print, "<ExtInt line=%u>", self->line);
 }
 
@@ -535,7 +583,7 @@ void Handle_EXTI_Irq(uint32_t line) {
                     *cb = mp_const_none;
                     extint_disable(line);
                     printf("Uncaught exception in ExtInt interrupt handler line %u\n", (unsigned int)line);
-                    mp_obj_print_exception(&mp_plat_print, (mp_obj_t)nlr.ret_val);
+                    mp_obj_print_exception(&mp_plat_print, MP_OBJ_FROM_PTR(nlr.ret_val));
                 }
                 gc_unlock();
                 mp_sched_unlock();
