@@ -37,10 +37,15 @@
 #include "extmod/vfs.h"
 #include "extmod/vfs_fat.h"
 
+#if MICROPY_PY_LWIP
+#include "lwip/init.h"
+#endif
+
 #include "systick.h"
 #include "pendsv.h"
 #include "pybthread.h"
 #include "gccollect.h"
+#include "factoryreset.h"
 #include "modmachine.h"
 #include "i2c.h"
 #include "spi.h"
@@ -54,6 +59,7 @@
 #include "rtc.h"
 #include "storage.h"
 #include "sdcard.h"
+#include "sdram.h"
 #include "rng.h"
 #include "accel.h"
 #include "servo.h"
@@ -63,8 +69,21 @@
 
 void SystemClock_Config(void);
 
-pyb_thread_t pyb_thread_main;
-fs_user_mount_t fs_user_mount_flash;
+#if MICROPY_PY_THREAD
+STATIC pyb_thread_t pyb_thread_main;
+#endif
+
+#if MICROPY_HW_ENABLE_STORAGE
+STATIC fs_user_mount_t fs_user_mount_flash;
+#endif
+
+#if defined(MICROPY_HW_UART_REPL)
+#ifndef MICROPY_HW_UART_REPL_RXBUF
+#define MICROPY_HW_UART_REPL_RXBUF (260)
+#endif
+STATIC pyb_uart_obj_t pyb_uart_repl_obj;
+STATIC uint8_t pyb_uart_repl_rxbuf[MICROPY_HW_UART_REPL_RXBUF];
+#endif
 
 void flash_error(int n) {
     for (int i = 0; i < n; i++) {
@@ -100,7 +119,7 @@ void NORETURN __fatal_error(const char *msg) {
 
 void nlr_jump_fail(void *val) {
     printf("FATAL: uncaught exception %p\n", val);
-    mp_obj_print_exception(&mp_plat_print, (mp_obj_t)val);
+    mp_obj_print_exception(&mp_plat_print, MP_OBJ_FROM_PTR(val));
     __fatal_error("");
 }
 
@@ -117,7 +136,7 @@ STATIC mp_obj_t pyb_main(size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_a
         { MP_QSTR_opt, MP_ARG_INT, {.u_int = 0} }
     };
 
-    if (MP_OBJ_IS_STR(pos_args[0])) {
+    if (mp_obj_is_str(pos_args[0])) {
         MP_STATE_PORT(pyb_config_main) = pos_args[0];
 
         // parse args
@@ -129,42 +148,7 @@ STATIC mp_obj_t pyb_main(size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_a
 }
 MP_DEFINE_CONST_FUN_OBJ_KW(pyb_main_obj, 1, pyb_main);
 
-static const char fresh_boot_py[] =
-"# boot.py -- run on boot-up\r\n"
-"# can run arbitrary Python, but best to keep it minimal\r\n"
-"\r\n"
-"import machine\r\n"
-"import pyb\r\n"
-"#pyb.main('main.py') # main script to run after this one\r\n"
-#if MICROPY_HW_ENABLE_USB
-"#pyb.usb_mode('VCP+MSC') # act as a serial and a storage device\r\n"
-"#pyb.usb_mode('VCP+HID') # act as a serial device and a mouse\r\n"
-#endif
-;
-
-static const char fresh_main_py[] =
-"# main.py -- put your code here!\r\n"
-;
-
-static const char fresh_pybcdc_inf[] =
-#include "genhdr/pybcdc_inf.h"
-;
-
-static const char fresh_readme_txt[] =
-"This is a MicroPython board\r\n"
-"\r\n"
-"You can get started right away by writing your Python code in 'main.py'.\r\n"
-"\r\n"
-"For a serial prompt:\r\n"
-" - Windows: you need to go to 'Device manager', right click on the unknown device,\r\n"
-"   then update the driver software, using the 'pybcdc.inf' file found on this drive.\r\n"
-"   Then use a terminal program like Hyperterminal or putty.\r\n"
-" - Mac OS X: use the command: screen /dev/tty.usbmodem*\r\n"
-" - Linux: use the command: screen /dev/ttyACM0\r\n"
-"\r\n"
-"Please visit http://micropython.org/help/ for further help.\r\n"
-;
-
+#if MICROPY_HW_ENABLE_STORAGE
 // avoid inlining to avoid stack usage within main()
 MP_NOINLINE STATIC bool init_flash_fs(uint reset_mode) {
     // init the vfs object
@@ -182,44 +166,29 @@ MP_NOINLINE STATIC bool init_flash_fs(uint reset_mode) {
         led_state(PYB_LED_GREEN, 1);
         uint32_t start_tick = HAL_GetTick();
 
-        uint8_t working_buf[_MAX_SS];
+        uint8_t working_buf[FF_MAX_SS];
         res = f_mkfs(&vfs_fat->fatfs, FM_FAT, 0, working_buf, sizeof(working_buf));
         if (res == FR_OK) {
             // success creating fresh LFS
         } else {
-            printf("PYB: can't create flash filesystem\n");
+            printf("MPY: can't create flash filesystem\n");
             return false;
         }
 
         // set label
         f_setlabel(&vfs_fat->fatfs, MICROPY_HW_FLASH_FS_LABEL);
 
-        // create empty main.py
-        FIL fp;
-        f_open(&vfs_fat->fatfs, &fp, "/main.py", FA_WRITE | FA_CREATE_ALWAYS);
-        UINT n;
-        f_write(&fp, fresh_main_py, sizeof(fresh_main_py) - 1 /* don't count null terminator */, &n);
-        // TODO check we could write n bytes
-        f_close(&fp);
-
-        // create .inf driver file
-        f_open(&vfs_fat->fatfs, &fp, "/pybcdc.inf", FA_WRITE | FA_CREATE_ALWAYS);
-        f_write(&fp, fresh_pybcdc_inf, sizeof(fresh_pybcdc_inf) - 1 /* don't count null terminator */, &n);
-        f_close(&fp);
-
-        // create readme file
-        f_open(&vfs_fat->fatfs, &fp, "/README.txt", FA_WRITE | FA_CREATE_ALWAYS);
-        f_write(&fp, fresh_readme_txt, sizeof(fresh_readme_txt) - 1 /* don't count null terminator */, &n);
-        f_close(&fp);
+        // populate the filesystem with factory files
+        factory_reset_make_files(&vfs_fat->fatfs);
 
         // keep LED on for at least 200ms
-        sys_tick_wait_at_least(start_tick, 200);
+        systick_wait_at_least(start_tick, 200);
         led_state(PYB_LED_GREEN, 0);
     } else if (res == FR_OK) {
         // mount sucessful
     } else {
     fail:
-        printf("PYB: can't mount flash\n");
+        printf("MPY: can't mount flash\n");
         return false;
     }
 
@@ -239,32 +208,11 @@ MP_NOINLINE STATIC bool init_flash_fs(uint reset_mode) {
     // It is set to the internal flash filesystem by default.
     MP_STATE_PORT(vfs_cur) = vfs;
 
-    // Make sure we have a /flash/boot.py.  Create it if needed.
-    FILINFO fno;
-    res = f_stat(&vfs_fat->fatfs, "/boot.py", &fno);
-    if (res != FR_OK) {
-        // doesn't exist, create fresh file
-
-        // LED on to indicate creation of boot.py
-        led_state(PYB_LED_GREEN, 1);
-        uint32_t start_tick = HAL_GetTick();
-
-        FIL fp;
-        f_open(&vfs_fat->fatfs, &fp, "/boot.py", FA_WRITE | FA_CREATE_ALWAYS);
-        UINT n;
-        f_write(&fp, fresh_boot_py, sizeof(fresh_boot_py) - 1 /* don't count null terminator */, &n);
-        // TODO check we could write n bytes
-        f_close(&fp);
-
-        // keep LED on for at least 200ms
-        sys_tick_wait_at_least(start_tick, 200);
-        led_state(PYB_LED_GREEN, 0);
-    }
-
     return true;
 }
+#endif
 
-#if MICROPY_HW_HAS_SDCARD
+#if MICROPY_HW_SDCARD_MOUNT_AT_BOOT
 STATIC bool init_sdcard_fs(void) {
     bool first_part = true;
     for (int part_num = 1; part_num <= 4; ++part_num) {
@@ -332,7 +280,7 @@ STATIC bool init_sdcard_fs(void) {
     }
 
     if (first_part) {
-        printf("PYB: can't mount SD card\n");
+        printf("MPY: can't mount SD card\n");
         return false;
     } else {
         return true;
@@ -342,12 +290,12 @@ STATIC bool init_sdcard_fs(void) {
 
 #if !MICROPY_HW_USES_BOOTLOADER
 STATIC uint update_reset_mode(uint reset_mode) {
-#if MICROPY_HW_HAS_SWITCH
+    #if MICROPY_HW_HAS_SWITCH
     if (switch_get()) {
 
         // The original method used on the pyboard is appropriate if you have 2
         // or more LEDs.
-#if defined(MICROPY_HW_LED2)
+        #if defined(MICROPY_HW_LED2)
         for (uint i = 0; i < 3000; i++) {
             if (!switch_get()) {
                 break;
@@ -375,7 +323,7 @@ STATIC uint update_reset_mode(uint reset_mode) {
         }
         mp_hal_delay_ms(400);
 
-#elif defined(MICROPY_HW_LED1)
+        #elif defined(MICROPY_HW_LED1)
 
         // For boards with only a single LED, we'll flash that LED the
         // appropriate number of times, with a pause between each one
@@ -408,11 +356,11 @@ STATIC uint update_reset_mode(uint reset_mode) {
             }
             mp_hal_delay_ms(400);
         }
-#else
-#error Need a reset mode update method
-#endif
+        #else
+        #error Need a reset mode update method
+        #endif
     }
-#endif
+    #endif
     return reset_mode;
 }
 #endif
@@ -455,8 +403,10 @@ void stm32_main(uint32_t reset_mode) {
 
     #endif
 
+    #if __CORTEX_M >= 0x03
     // Set the priority grouping
     NVIC_SetPriorityGrouping(NVIC_PRIORITYGROUP_4);
+    #endif
 
     // SysTick is needed by HAL_RCC_ClockConfig (called in SystemClock_Config)
     HAL_InitTick(TICK_INT_PRIORITY);
@@ -468,7 +418,9 @@ void stm32_main(uint32_t reset_mode) {
     __HAL_RCC_GPIOA_CLK_ENABLE();
     __HAL_RCC_GPIOB_CLK_ENABLE();
     __HAL_RCC_GPIOC_CLK_ENABLE();
+    #if defined(GPIOD)
     __HAL_RCC_GPIOD_CLK_ENABLE();
+    #endif
 
     #if defined(STM32F4) ||  defined(STM32F7)
         #if defined(__HAL_RCC_DTCMRAMEN_CLK_ENABLE)
@@ -492,6 +444,12 @@ void stm32_main(uint32_t reset_mode) {
     #endif
 
     // basic sub-system init
+    #if MICROPY_HW_SDRAM_SIZE
+    sdram_init();
+    #if MICROPY_HW_SDRAM_STARTUP_TEST
+    sdram_test(true);
+    #endif
+    #endif
     #if MICROPY_PY_THREAD
     pyb_thread_init(&pyb_thread_main);
     #endif
@@ -504,24 +462,47 @@ void stm32_main(uint32_t reset_mode) {
     #if MICROPY_HW_ENABLE_RTC
     rtc_init_start(false);
     #endif
+    uart_init0();
     spi_init0();
-    #if MICROPY_HW_ENABLE_HW_I2C
+    #if MICROPY_PY_PYB_LEGACY && MICROPY_HW_ENABLE_HW_I2C
     i2c_init0();
     #endif
-    #if MICROPY_HW_HAS_SDCARD
+    #if MICROPY_HW_ENABLE_SDCARD
     sdcard_init();
     #endif
+    #if MICROPY_HW_ENABLE_STORAGE
     storage_init();
+    #endif
+    #if MICROPY_PY_LWIP
+    // lwIP doesn't allow to reinitialise itself by subsequent calls to this function
+    // because the system timeout list (next_timeout) is only ever reset by BSS clearing.
+    // So for now we only init the lwIP stack once on power-up.
+    lwip_init();
+    systick_enable_dispatch(SYSTICK_DISPATCH_LWIP, mod_network_lwip_poll_wrapper);
+    #endif
+
+    #if defined(MICROPY_HW_UART_REPL)
+    // Set up a UART REPL using a statically allocated object
+    pyb_uart_repl_obj.base.type = &pyb_uart_type;
+    pyb_uart_repl_obj.uart_id = MICROPY_HW_UART_REPL;
+    pyb_uart_repl_obj.is_static = true;
+    pyb_uart_repl_obj.timeout = 0;
+    pyb_uart_repl_obj.timeout_char = 2;
+    uart_init(&pyb_uart_repl_obj, MICROPY_HW_UART_REPL_BAUD, UART_WORDLENGTH_8B, UART_PARITY_NONE, UART_STOPBITS_1, 0);
+    uart_set_rxbuf(&pyb_uart_repl_obj, sizeof(pyb_uart_repl_rxbuf), pyb_uart_repl_rxbuf);
+    uart_attach_to_repl(&pyb_uart_repl_obj, true);
+    MP_STATE_PORT(pyb_uart_obj_all)[MICROPY_HW_UART_REPL - 1] = &pyb_uart_repl_obj;
+    #endif
 
 soft_reset:
 
-#if defined(MICROPY_HW_LED2)
+    #if defined(MICROPY_HW_LED2)
     led_state(1, 0);
     led_state(2, 1);
-#else
+    #else
     led_state(1, 1);
     led_state(2, 0);
-#endif
+    #endif
     led_state(3, 0);
     led_state(4, 0);
 
@@ -542,7 +523,7 @@ soft_reset:
     mp_stack_set_limit((char*)&_estack - (char*)&_heap_end - 1024);
 
     // GC init
-    gc_init(&_heap_start, &_heap_end);
+    gc_init(MICROPY_HEAP_START, MICROPY_HEAP_END);
 
     #if MICROPY_ENABLE_PYSTACK
     static mp_obj_t pystack[384];
@@ -551,51 +532,47 @@ soft_reset:
 
     // MicroPython init
     mp_init();
-    mp_obj_list_init(mp_sys_path, 0);
+    mp_obj_list_init(MP_OBJ_TO_PTR(mp_sys_path), 0);
     mp_obj_list_append(mp_sys_path, MP_OBJ_NEW_QSTR(MP_QSTR_)); // current dir (or base dir of the script)
-    mp_obj_list_init(mp_sys_argv, 0);
+    mp_obj_list_init(MP_OBJ_TO_PTR(mp_sys_argv), 0);
 
     // Initialise low-level sub-systems.  Here we need to very basic things like
     // zeroing out memory and resetting any of the sub-systems.  Following this
     // we can run Python scripts (eg boot.py), but anything that is configurable
     // by boot.py must be set after boot.py is run.
 
+    #if defined(MICROPY_HW_UART_REPL)
+    MP_STATE_PORT(pyb_stdio_uart) = &pyb_uart_repl_obj;
+    #else
+    MP_STATE_PORT(pyb_stdio_uart) = NULL;
+    #endif
+
     readline_init0();
     pin_init0();
     extint_init0();
     timer_init0();
-    uart_init0();
 
-    // Define MICROPY_HW_UART_REPL to be PYB_UART_6 and define
-    // MICROPY_HW_UART_REPL_BAUD in your mpconfigboard.h file if you want a
-    // REPL on a hardware UART as well as on USB VCP
-#if defined(MICROPY_HW_UART_REPL)
-    {
-        mp_obj_t args[2] = {
-            MP_OBJ_NEW_SMALL_INT(MICROPY_HW_UART_REPL),
-            MP_OBJ_NEW_SMALL_INT(MICROPY_HW_UART_REPL_BAUD),
-        };
-        MP_STATE_PORT(pyb_stdio_uart) = pyb_uart_type.make_new((mp_obj_t)&pyb_uart_type, MP_ARRAY_SIZE(args), 0, args);
-        uart_attach_to_repl(MP_STATE_PORT(pyb_stdio_uart), true);
-    }
-#else
-    MP_STATE_PORT(pyb_stdio_uart) = NULL;
-#endif
-
-#if MICROPY_HW_ENABLE_CAN
+    #if MICROPY_HW_ENABLE_CAN
     can_init0();
-#endif
+    #endif
 
     #if MICROPY_HW_ENABLE_USB
     pyb_usb_init0();
+
+    // Activate USB_VCP(0) on dupterm slot 1 for the REPL
+    MP_STATE_VM(dupterm_objs[1]) = MP_OBJ_FROM_PTR(&pyb_usb_vcp_obj);
+    usb_vcp_attach_to_repl(&pyb_usb_vcp_obj, true);
     #endif
 
     // Initialise the local flash filesystem.
     // Create it if needed, mount in on /flash, and set it as current dir.
-    bool mounted_flash = init_flash_fs(reset_mode);
+    bool mounted_flash = false;
+    #if MICROPY_HW_ENABLE_STORAGE
+    mounted_flash = init_flash_fs(reset_mode);
+    #endif
 
     bool mounted_sdcard = false;
-#if MICROPY_HW_HAS_SDCARD
+    #if MICROPY_HW_SDCARD_MOUNT_AT_BOOT
     // if an SD card is present then mount it on /sd/
     if (sdcard_is_present()) {
         // if there is a file in the flash called "SKIPSD", then we don't mount the SD card
@@ -603,7 +580,7 @@ soft_reset:
             mounted_sdcard = init_sdcard_fs();
         }
     }
-#endif
+    #endif
 
     #if MICROPY_HW_ENABLE_USB
     // if the SD card isn't used as the USB MSC medium then use the internal flash
@@ -629,25 +606,22 @@ soft_reset:
     // TODO perhaps have pyb.reboot([bootpy]) function to soft-reboot and execute custom boot.py
     if (reset_mode == 1 || reset_mode == 3) {
         const char *boot_py = "boot.py";
-        mp_import_stat_t stat = mp_import_stat(boot_py);
-        if (stat == MP_IMPORT_STAT_FILE) {
-            int ret = pyexec_file(boot_py);
-            if (ret & PYEXEC_FORCED_EXIT) {
-                goto soft_reset_exit;
-            }
-            if (!ret) {
-                flash_error(4);
-            }
+        int ret = pyexec_file_if_exists(boot_py);
+        if (ret & PYEXEC_FORCED_EXIT) {
+            goto soft_reset_exit;
+        }
+        if (!ret) {
+            flash_error(4);
         }
     }
 
     // turn boot-up LEDs off
-#if !defined(MICROPY_HW_LED2)
+    #if !defined(MICROPY_HW_LED2)
     // If there is only one LED on the board then it's used to signal boot-up
     // and so we turn it off here.  Otherwise LED(1) is used to indicate dirty
     // flash cache and so we shouldn't change its state.
     led_state(1, 0);
-#endif
+    #endif
     led_state(2, 0);
     led_state(3, 0);
     led_state(4, 0);
@@ -663,24 +637,18 @@ soft_reset:
     }
     #endif
 
-#if MICROPY_HW_HAS_MMA7660
+    #if MICROPY_HW_HAS_MMA7660
     // MMA accel: init and reset
     accel_init();
-#endif
+    #endif
 
-#if MICROPY_HW_ENABLE_SERVO
-    // servo
+    #if MICROPY_HW_ENABLE_SERVO
     servo_init();
-#endif
+    #endif
 
-#if MICROPY_HW_ENABLE_DAC
-    // DAC
-    dac_init();
-#endif
-
-#if MICROPY_PY_NETWORK
+    #if MICROPY_PY_NETWORK
     mod_network_init();
-#endif
+    #endif
 
     // At this point everything is fully configured and initialised.
 
@@ -692,15 +660,12 @@ soft_reset:
         } else {
             main_py = mp_obj_str_get_str(MP_STATE_PORT(pyb_config_main));
         }
-        mp_import_stat_t stat = mp_import_stat(main_py);
-        if (stat == MP_IMPORT_STAT_FILE) {
-            int ret = pyexec_file(main_py);
-            if (ret & PYEXEC_FORCED_EXIT) {
-                goto soft_reset_exit;
-            }
-            if (!ret) {
-                flash_error(3);
-            }
+        int ret = pyexec_file_if_exists(main_py);
+        if (ret & PYEXEC_FORCED_EXIT) {
+            goto soft_reset_exit;
+        }
+        if (!ret) {
+            flash_error(3);
         }
     }
 
@@ -722,20 +687,27 @@ soft_reset_exit:
 
     // soft reset
 
-    printf("PYB: sync filesystems\n");
+    #if MICROPY_HW_ENABLE_STORAGE
+    printf("MPY: sync filesystems\n");
     storage_flush();
+    #endif
 
-    printf("PYB: soft reboot\n");
+    printf("MPY: soft reboot\n");
+    #if MICROPY_PY_NETWORK
+    mod_network_deinit();
+    #endif
     timer_deinit();
-    uart_deinit();
-#if MICROPY_HW_ENABLE_CAN
+    uart_deinit_all();
+    #if MICROPY_HW_ENABLE_CAN
     can_deinit();
-#endif
+    #endif
     machine_deinit();
 
     #if MICROPY_PY_THREAD
     pyb_thread_deinit();
     #endif
 
+    gc_sweep_all();
+
     goto soft_reset;
 }
