@@ -30,6 +30,8 @@
 #include "modmachine.h"
 #include "py/gc.h"
 #include "py/runtime.h"
+#include "py/objstr.h"
+#include "py/mperrno.h"
 #include "py/mphal.h"
 #include "extmod/machine_mem.h"
 #include "extmod/machine_signal.h"
@@ -41,6 +43,7 @@
 #include "extmod/vfs_fat.h"
 #include "gccollect.h"
 #include "irq.h"
+#include "powerctrl.h"
 #include "pybthread.h"
 #include "rng.h"
 #include "storage.h"
@@ -52,7 +55,6 @@
 #include "spi.h"
 #include "uart.h"
 #include "wdt.h"
-#include "genhdr/pllfreqtable.h"
 
 #if defined(STM32L4)
 // L4 does not have a POR, so use BOR instead
@@ -104,13 +106,23 @@ void machine_init(void) {
         reset_cause = PYB_RESET_DEEPSLEEP;
         PWR->CPUCR |= PWR_CPUCR_CSSF;
     } else
+    #elif defined(STM32L4)
+    if (PWR->SR1 & PWR_SR1_SBF) {
+        // came out of standby
+        reset_cause = PYB_RESET_DEEPSLEEP;
+        PWR->SCR |= PWR_SCR_CSBF;
+    } else
     #endif
     {
         // get reset cause from RCC flags
         uint32_t state = RCC->RCC_SR;
         if (state & RCC_SR_IWDGRSTF || state & RCC_SR_WWDGRSTF) {
             reset_cause = PYB_RESET_WDT;
-        } else if (state & RCC_SR_PORRSTF || state & RCC_SR_BORRSTF) {
+        } else if (state & RCC_SR_PORRSTF
+            #if !defined(STM32F0)
+            || state & RCC_SR_BORRSTF
+            #endif
+            ) {
             reset_cause = PYB_RESET_POWER_ON;
         } else if (state & RCC_SR_PINRSTF) {
             reset_cause = PYB_RESET_HARD;
@@ -140,11 +152,18 @@ STATIC mp_obj_t machine_info(size_t n_args, const mp_obj_t *args) {
     // get and print clock speeds
     // SYSCLK=168MHz, HCLK=168MHz, PCLK1=42MHz, PCLK2=84MHz
     {
+        #if defined(STM32F0)
+        printf("S=%u\nH=%u\nP1=%u\n",
+               (unsigned int)HAL_RCC_GetSysClockFreq(),
+               (unsigned int)HAL_RCC_GetHCLKFreq(),
+               (unsigned int)HAL_RCC_GetPCLK1Freq());
+        #else
         printf("S=%u\nH=%u\nP1=%u\nP2=%u\n",
                (unsigned int)HAL_RCC_GetSysClockFreq(),
                (unsigned int)HAL_RCC_GetHCLKFreq(),
                (unsigned int)HAL_RCC_GetPCLK1Freq(),
                (unsigned int)HAL_RCC_GetPCLK2Freq());
+        #endif
     }
 
     // to print info about memory
@@ -164,9 +183,9 @@ STATIC mp_obj_t machine_info(size_t n_args, const mp_obj_t *args) {
 
     // qstr info
     {
-        mp_uint_t n_pool, n_qstr, n_str_data_bytes, n_total_bytes;
+        size_t n_pool, n_qstr, n_str_data_bytes, n_total_bytes;
         qstr_pool_info(&n_pool, &n_qstr, &n_str_data_bytes, &n_total_bytes);
-        printf("qstr:\n  n_pool=" UINT_FMT "\n  n_qstr=" UINT_FMT "\n  n_str_data_bytes=" UINT_FMT "\n  n_total_bytes=" UINT_FMT "\n", n_pool, n_qstr, n_str_data_bytes, n_total_bytes);
+        printf("qstr:\n  n_pool=%u\n  n_qstr=%u\n  n_str_data_bytes=%u\n  n_total_bytes=%u\n", n_pool, n_qstr, n_str_data_bytes, n_total_bytes);
     }
 
     // GC info
@@ -174,13 +193,14 @@ STATIC mp_obj_t machine_info(size_t n_args, const mp_obj_t *args) {
         gc_info_t info;
         gc_info(&info);
         printf("GC:\n");
-        printf("  " UINT_FMT " total\n", info.total);
-        printf("  " UINT_FMT " : " UINT_FMT "\n", info.used, info.free);
-        printf("  1=" UINT_FMT " 2=" UINT_FMT " m=" UINT_FMT "\n", info.num_1block, info.num_2block, info.max_block);
+        printf("  %u total\n", info.total);
+        printf("  %u : %u\n", info.used, info.free);
+        printf("  1=%u 2=%u m=%u\n", info.num_1block, info.num_2block, info.max_block);
     }
 
     // free space on flash
     {
+        #if MICROPY_VFS_FAT
         for (mp_vfs_mount_t *vfs = MP_STATE_VM(vfs_mount_table); vfs != NULL; vfs = vfs->next) {
             if (strncmp("/flash", vfs->str, vfs->len) == 0) {
                 // assumes that it's a FatFs filesystem
@@ -191,6 +211,7 @@ STATIC mp_obj_t machine_info(size_t n_args, const mp_obj_t *args) {
                 break;
             }
         }
+        #endif
     }
 
     #if MICROPY_PY_THREAD
@@ -226,12 +247,29 @@ STATIC mp_obj_t machine_soft_reset(void) {
 }
 MP_DEFINE_CONST_FUN_OBJ_0(machine_soft_reset_obj, machine_soft_reset);
 
+__attribute__((naked)) void branch_to_bootloader(uint32_t r0, uint32_t addr) {
+    __asm volatile (
+        "ldr r2, [r1, #0]\n"    // get address of stack pointer
+        "msr msp, r2\n"         // get stack pointer
+        "ldr r2, [r1, #4]\n"    // get address of destination
+        "bx r2\n"               // branch to bootloader
+    );
+}
+
 // Activate the bootloader without BOOT* pins.
-STATIC NORETURN mp_obj_t machine_bootloader(void) {
+STATIC NORETURN mp_obj_t machine_bootloader(size_t n_args, const mp_obj_t *args) {
     #if MICROPY_HW_ENABLE_USB
     pyb_usb_dev_deinit();
     #endif
+    #if MICROPY_HW_ENABLE_STORAGE
     storage_flush();
+    #endif
+
+    #if __DCACHE_PRESENT == 1
+    // Flush and disable caches before turning off peripherals (eg SDRAM)
+    SCB_DisableICache();
+    SCB_DisableDCache();
+    #endif
 
     HAL_RCC_DeInit();
     HAL_DeInit();
@@ -241,336 +279,96 @@ STATIC NORETURN mp_obj_t machine_bootloader(void) {
     HAL_MPU_Disable();
     #endif
 
-#if defined(STM32F7) || defined(STM32H7)
-    // arm-none-eabi-gcc 4.9.0 does not correctly inline this
-    // MSP function, so we write it out explicitly here.
-    //__set_MSP(*((uint32_t*) 0x1FF00000));
-    __ASM volatile ("movw r3, #0x0000\nmovt r3, #0x1FF0\nldr r3, [r3, #0]\nMSR msp, r3\n" : : : "r3", "sp");
+    #if MICROPY_HW_USES_BOOTLOADER
+    if (n_args == 0 || !mp_obj_is_true(args[0])) {
+        // By default, with no args given, we enter the custom bootloader (mboot)
+        branch_to_bootloader(0x70ad0000, 0x08000000);
+    }
 
-    ((void (*)(void)) *((uint32_t*) 0x1FF00004))();
-#else
+    if (n_args == 1 && mp_obj_is_str_or_bytes(args[0])) {
+        // With a string/bytes given, pass its data to the custom bootloader
+        size_t len;
+        const char *data = mp_obj_str_get_data(args[0], &len);
+        void *mboot_region = (void*)*((volatile uint32_t*)0x08000000);
+        memmove(mboot_region, data, len);
+        branch_to_bootloader(0x70ad0080, 0x08000000);
+    }
+    #endif
+
+    #if defined(STM32F7) || defined(STM32H7)
+    branch_to_bootloader(0, 0x1ff00000);
+    #else
     __HAL_SYSCFG_REMAPMEMORY_SYSTEMFLASH();
-
-    // arm-none-eabi-gcc 4.9.0 does not correctly inline this
-    // MSP function, so we write it out explicitly here.
-    //__set_MSP(*((uint32_t*) 0x00000000));
-    __ASM volatile ("movs r3, #0\nldr r3, [r3, #0]\nMSR msp, r3\n" : : : "r3", "sp");
-
-    ((void (*)(void)) *((uint32_t*) 0x00000004))();
-#endif
+    branch_to_bootloader(0, 0x00000000);
+    #endif
 
     while (1);
 }
-MP_DEFINE_CONST_FUN_OBJ_0(machine_bootloader_obj, machine_bootloader);
+MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(machine_bootloader_obj, 0, 1, machine_bootloader);
 
 // get or set the MCU frequencies
-STATIC mp_uint_t machine_freq_calc_ahb_div(mp_uint_t wanted_div) {
-    if (wanted_div <= 1) { return RCC_SYSCLK_DIV1; }
-    else if (wanted_div <= 2) { return RCC_SYSCLK_DIV2; }
-    else if (wanted_div <= 4) { return RCC_SYSCLK_DIV4; }
-    else if (wanted_div <= 8) { return RCC_SYSCLK_DIV8; }
-    else if (wanted_div <= 16) { return RCC_SYSCLK_DIV16; }
-    else if (wanted_div <= 64) { return RCC_SYSCLK_DIV64; }
-    else if (wanted_div <= 128) { return RCC_SYSCLK_DIV128; }
-    else if (wanted_div <= 256) { return RCC_SYSCLK_DIV256; }
-    else { return RCC_SYSCLK_DIV512; }
-}
-STATIC mp_uint_t machine_freq_calc_apb_div(mp_uint_t wanted_div) {
-    if (wanted_div <= 1) { return RCC_HCLK_DIV1; }
-    else if (wanted_div <= 2) { return RCC_HCLK_DIV2; }
-    else if (wanted_div <= 4) { return RCC_HCLK_DIV4; }
-    else if (wanted_div <= 8) { return RCC_HCLK_DIV8; }
-    else { return RCC_SYSCLK_DIV16; }
-}
 STATIC mp_obj_t machine_freq(size_t n_args, const mp_obj_t *args) {
     if (n_args == 0) {
         // get
-        mp_obj_t tuple[4] = {
+        mp_obj_t tuple[] = {
            mp_obj_new_int(HAL_RCC_GetSysClockFreq()),
            mp_obj_new_int(HAL_RCC_GetHCLKFreq()),
            mp_obj_new_int(HAL_RCC_GetPCLK1Freq()),
+           #if !defined(STM32F0)
            mp_obj_new_int(HAL_RCC_GetPCLK2Freq()),
+           #endif
         };
-        return mp_obj_new_tuple(4, tuple);
+        return mp_obj_new_tuple(MP_ARRAY_SIZE(tuple), tuple);
     } else {
         // set
-        mp_int_t wanted_sysclk = mp_obj_get_int(args[0]) / 1000000;
-
-        #if defined(STM32L4)
+        #if defined(STM32F0) || defined(STM32L4)
         mp_raise_NotImplementedError("machine.freq set not supported yet");
-        #endif
-
-        // default PLL parameters that give 48MHz on PLL48CK
-        uint32_t m = HSE_VALUE / 1000000, n = 336, p = 2, q = 7;
-        uint32_t sysclk_source;
-
-        // search for a valid PLL configuration that keeps USB at 48MHz
-        for (const uint16_t *pll = &pll_freq_table[MP_ARRAY_SIZE(pll_freq_table) - 1]; pll >= &pll_freq_table[0]; --pll) {
-            uint32_t sys = *pll & 0xff;
-            if (sys <= wanted_sysclk) {
-                m = (*pll >> 10) & 0x3f;
-                p = ((*pll >> 7) & 0x6) + 2;
-                if (m == 0) {
-                    // special entry for using HSI directly
-                    sysclk_source = RCC_SYSCLKSOURCE_HSI;
-                    goto set_clk;
-                } else if (m == 1) {
-                    // special entry for using HSE directly
-                    sysclk_source = RCC_SYSCLKSOURCE_HSE;
-                    goto set_clk;
-                } else {
-                    // use PLL
-                    sysclk_source = RCC_SYSCLKSOURCE_PLLCLK;
-                    uint32_t vco_out = sys * p;
-                    n = vco_out * m / (HSE_VALUE / 1000000);
-                    q = vco_out / 48;
-                    goto set_clk;
+        #else
+        mp_int_t sysclk = mp_obj_get_int(args[0]);
+        mp_int_t ahb = sysclk;
+        mp_int_t apb1 = ahb / 4;
+        mp_int_t apb2 = ahb / 2;
+        if (n_args > 1) {
+            ahb = mp_obj_get_int(args[1]);
+            if (n_args > 2) {
+                apb1 = mp_obj_get_int(args[2]);
+                if (n_args > 3) {
+                    apb2 = mp_obj_get_int(args[3]);
                 }
             }
         }
-        mp_raise_ValueError("can't make valid freq");
-
-    set_clk:
-        //printf("%lu %lu %lu %lu %lu\n", sysclk_source, m, n, p, q);
-
-        // let the USB CDC have a chance to process before we change the clock
-        mp_hal_delay_ms(5);
-
-        // desired system clock source is in sysclk_source
-        RCC_ClkInitTypeDef RCC_ClkInitStruct;
-        RCC_ClkInitStruct.ClockType = (RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2);
-        if (sysclk_source == RCC_SYSCLKSOURCE_PLLCLK) {
-            // set HSE as system clock source to allow modification of the PLL configuration
-            // we then change to PLL after re-configuring PLL
-            RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_HSE;
-        } else {
-            // directly set the system clock source as desired
-            RCC_ClkInitStruct.SYSCLKSource = sysclk_source;
+        int ret = powerctrl_set_sysclk(sysclk, ahb, apb1, apb2);
+        if (ret == -MP_EINVAL) {
+            mp_raise_ValueError("invalid freq");
+        } else if (ret < 0) {
+            void NORETURN __fatal_error(const char *msg);
+            __fatal_error("can't change freq");
         }
-        wanted_sysclk *= 1000000;
-        if (n_args >= 2) {
-            // note: AHB freq required to be >= 14.2MHz for USB operation
-            RCC_ClkInitStruct.AHBCLKDivider = machine_freq_calc_ahb_div(wanted_sysclk / mp_obj_get_int(args[1]));
-        } else {
-            RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
-        }
-        if (n_args >= 3) {
-            RCC_ClkInitStruct.APB1CLKDivider = machine_freq_calc_apb_div(wanted_sysclk / mp_obj_get_int(args[2]));
-        } else {
-            RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
-        }
-        if (n_args >= 4) {
-            RCC_ClkInitStruct.APB2CLKDivider = machine_freq_calc_apb_div(wanted_sysclk / mp_obj_get_int(args[3]));
-        } else {
-            RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;
-        }
-        #if defined(MICROPY_HW_CLK_LAST_FREQ) && MICROPY_HW_CLK_LAST_FREQ
-        uint32_t h = RCC_ClkInitStruct.AHBCLKDivider >> 4;
-        uint32_t b1 = RCC_ClkInitStruct.APB1CLKDivider >> 10;
-        uint32_t b2 = RCC_ClkInitStruct.APB2CLKDivider >> 10;
-        #endif
-        if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_1) != HAL_OK) {
-            goto fail;
-        }
-
-        // re-configure PLL
-        // even if we don't use the PLL for the system clock, we still need it for USB, RNG and SDIO
-        RCC_OscInitTypeDef RCC_OscInitStruct;
-        RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
-        RCC_OscInitStruct.HSEState = MICROPY_HW_CLK_HSE_STATE;
-        RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
-        RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
-        RCC_OscInitStruct.PLL.PLLM = m;
-        RCC_OscInitStruct.PLL.PLLN = n;
-        RCC_OscInitStruct.PLL.PLLP = p;
-        RCC_OscInitStruct.PLL.PLLQ = q;
-        if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK) {
-            goto fail;
-        }
-
-        // set PLL as system clock source if wanted
-        if (sysclk_source == RCC_SYSCLKSOURCE_PLLCLK) {
-            uint32_t flash_latency;
-            #if defined(STM32F7)
-            // if possible, scale down the internal voltage regulator to save power
-            // the flash_latency values assume a supply voltage between 2.7V and 3.6V
-            uint32_t volt_scale;
-            if (wanted_sysclk <= 90000000) {
-                volt_scale = PWR_REGULATOR_VOLTAGE_SCALE3;
-                flash_latency = FLASH_LATENCY_2;
-            } else if (wanted_sysclk <= 120000000) {
-                volt_scale = PWR_REGULATOR_VOLTAGE_SCALE3;
-                flash_latency = FLASH_LATENCY_3;
-            } else if (wanted_sysclk <= 144000000) {
-                volt_scale = PWR_REGULATOR_VOLTAGE_SCALE3;
-                flash_latency = FLASH_LATENCY_4;
-            } else if (wanted_sysclk <= 180000000) {
-                volt_scale = PWR_REGULATOR_VOLTAGE_SCALE2;
-                flash_latency = FLASH_LATENCY_5;
-            } else if (wanted_sysclk <= 210000000) {
-                volt_scale = PWR_REGULATOR_VOLTAGE_SCALE1;
-                flash_latency = FLASH_LATENCY_6;
-            } else {
-                volt_scale = PWR_REGULATOR_VOLTAGE_SCALE1;
-                flash_latency = FLASH_LATENCY_7;
-            }
-            if (HAL_PWREx_ControlVoltageScaling(volt_scale) != HAL_OK) {
-                goto fail;
-            }
-            #endif
-
-            #if !defined(STM32F7)
-            #if !defined(MICROPY_HW_FLASH_LATENCY)
-            #define MICROPY_HW_FLASH_LATENCY FLASH_LATENCY_5
-            #endif
-            flash_latency = MICROPY_HW_FLASH_LATENCY;
-            #endif
-            RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_SYSCLK;
-            RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
-            if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, flash_latency) != HAL_OK) {
-                goto fail;
-            }
-        }
-
-        #if defined(MICROPY_HW_CLK_LAST_FREQ) && MICROPY_HW_CLK_LAST_FREQ
-        #if defined(STM32F7)
-        #define FREQ_BKP BKP31R
-        #else
-        #define FREQ_BKP BKP19R
-        #endif
-        // qqqqqqqq pppppppp nnnnnnnn nnmmmmmm
-        // qqqqQQQQ ppppppPP nNNNNNNN NNMMMMMM
-        // 222111HH HHQQQQPP nNNNNNNN NNMMMMMM
-        p = (p / 2) - 1;
-        RTC->FREQ_BKP = m
-            | (n << 6) | (p << 16) | (q << 18)
-            | (h << 22)
-            | (b1 << 26)
-            | (b2 << 29);
-        #endif
-
         return mp_const_none;
-
-    fail:;
-        void NORETURN __fatal_error(const char *msg);
-        __fatal_error("can't change freq");
+        #endif
     }
 }
 MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(machine_freq_obj, 0, 4, machine_freq);
 
-STATIC mp_obj_t machine_sleep(void) {
-    #if defined(STM32L4)
-
-    // Enter Stop 1 mode
-    __HAL_RCC_WAKEUPSTOP_CLK_CONFIG(RCC_STOP_WAKEUPCLOCK_MSI);
-    HAL_PWR_EnterSTOPMode(PWR_LOWPOWERREGULATOR_ON, PWR_STOPENTRY_WFI);
-
-    // reconfigure system clock after wakeup
-    // Enable Power Control clock
-    __HAL_RCC_PWR_CLK_ENABLE();
-
-    // Get the Oscillators configuration according to the internal RCC registers
-    RCC_OscInitTypeDef RCC_OscInitStruct = {0};
-    HAL_RCC_GetOscConfig(&RCC_OscInitStruct);
-    RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_MSI;
-    RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
-    HAL_RCC_OscConfig(&RCC_OscInitStruct);
-
-    // Get the Clocks configuration according to the internal RCC registers
-    RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
-    uint32_t pFLatency = 0;
-    HAL_RCC_GetClockConfig(&RCC_ClkInitStruct, &pFLatency);
-
-    // Select PLL as system clock source and configure the HCLK, PCLK1 and PCLK2 clock dividers
-    RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_SYSCLK;
-    RCC_ClkInitStruct.SYSCLKSource  = RCC_SYSCLKSOURCE_PLLCLK;
-    HAL_RCC_ClockConfig(&RCC_ClkInitStruct, pFLatency);
-
-    #else
-
-    // takes longer to wake but reduces stop current
-    HAL_PWREx_EnableFlashPowerDown();
-
-    # if defined(STM32F7)
-    HAL_PWR_EnterSTOPMode((PWR_CR1_LPDS | PWR_CR1_LPUDS | PWR_CR1_FPDS | PWR_CR1_UDEN), PWR_STOPENTRY_WFI);
-    # else
-    HAL_PWR_EnterSTOPMode(PWR_LOWPOWERREGULATOR_ON, PWR_STOPENTRY_WFI);
-    #endif
-
-    // reconfigure the system clock after waking up
-
-    // enable HSE
-    __HAL_RCC_HSE_CONFIG(MICROPY_HW_CLK_HSE_STATE);
-    while (!__HAL_RCC_GET_FLAG(RCC_FLAG_HSERDY)) {
+STATIC mp_obj_t machine_lightsleep(size_t n_args, const mp_obj_t *args) {
+    if (n_args != 0) {
+        mp_obj_t args2[2] = {MP_OBJ_NULL, args[0]};
+        pyb_rtc_wakeup(2, args2);
     }
-
-    // enable PLL
-    __HAL_RCC_PLL_ENABLE();
-    while (!__HAL_RCC_GET_FLAG(RCC_FLAG_PLLRDY)) {
-    }
-
-    // select PLL as system clock source
-    MODIFY_REG(RCC->CFGR, RCC_CFGR_SW, RCC_SYSCLKSOURCE_PLLCLK);
-    #if defined(STM32H7)
-    while (__HAL_RCC_GET_SYSCLK_SOURCE() != RCC_CFGR_SWS_PLL1) {
-    #else
-    while (__HAL_RCC_GET_SYSCLK_SOURCE() != RCC_CFGR_SWS_PLL) {
-    #endif
-    }
-
-    #endif
-
+    powerctrl_enter_stop_mode();
     return mp_const_none;
 }
-MP_DEFINE_CONST_FUN_OBJ_0(machine_sleep_obj, machine_sleep);
+MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(machine_lightsleep_obj, 0, 1, machine_lightsleep);
 
-STATIC mp_obj_t machine_deepsleep(void) {
-    rtc_init_finalise();
-
-#if defined(STM32L4)
-    printf("machine.deepsleep not supported yet\n");
-#else
-    // We need to clear the PWR wake-up-flag before entering standby, since
-    // the flag may have been set by a previous wake-up event.  Furthermore,
-    // we need to disable the wake-up sources while clearing this flag, so
-    // that if a source is active it does actually wake the device.
-    // See section 5.3.7 of RM0090.
-
-    // Note: we only support RTC ALRA, ALRB, WUT and TS.
-    // TODO support TAMP and WKUP (PA0 external pin).
-    uint32_t irq_bits = RTC_CR_ALRAIE | RTC_CR_ALRBIE | RTC_CR_WUTIE | RTC_CR_TSIE;
-
-    // save RTC interrupts
-    uint32_t save_irq_bits = RTC->CR & irq_bits;
-
-    // disable RTC interrupts
-    RTC->CR &= ~irq_bits;
-
-    // clear RTC wake-up flags
-    RTC->ISR &= ~(RTC_ISR_ALRAF | RTC_ISR_ALRBF | RTC_ISR_WUTF | RTC_ISR_TSF);
-
-    #if defined(STM32F7)
-    // disable wake-up flags
-    PWR->CSR2 &= ~(PWR_CSR2_EWUP6 | PWR_CSR2_EWUP5 | PWR_CSR2_EWUP4 | PWR_CSR2_EWUP3 | PWR_CSR2_EWUP2 | PWR_CSR2_EWUP1);
-    // clear global wake-up flag
-    PWR->CR2 |= PWR_CR2_CWUPF6 | PWR_CR2_CWUPF5 | PWR_CR2_CWUPF4 | PWR_CR2_CWUPF3 | PWR_CR2_CWUPF2 | PWR_CR2_CWUPF1;
-    #elif defined(STM32H7)
-    // TODO
-    #else
-    // clear global wake-up flag
-    PWR->CR |= PWR_CR_CWUF;
-    #endif
-
-    // enable previously-enabled RTC interrupts
-    RTC->CR |= save_irq_bits;
-
-    // enter standby mode
-    HAL_PWR_EnterSTANDBYMode();
-    // we never return; MCU is reset on exit from standby
-#endif
+STATIC mp_obj_t machine_deepsleep(size_t n_args, const mp_obj_t *args) {
+    if (n_args != 0) {
+        mp_obj_t args2[2] = {MP_OBJ_NULL, args[0]};
+        pyb_rtc_wakeup(2, args2);
+    }
+    powerctrl_enter_standby_mode();
     return mp_const_none;
 }
-MP_DEFINE_CONST_FUN_OBJ_0(machine_deepsleep_obj, machine_deepsleep);
+MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(machine_deepsleep_obj, 0, 1, machine_deepsleep);
 
 STATIC mp_obj_t machine_reset_cause(void) {
     return MP_OBJ_NEW_SMALL_INT(reset_cause);
@@ -589,7 +387,8 @@ STATIC const mp_rom_map_elem_t machine_module_globals_table[] = {
     { MP_ROM_QSTR(MP_QSTR_rng),                 MP_ROM_PTR(&pyb_rng_get_obj) },
 #endif
     { MP_ROM_QSTR(MP_QSTR_idle),                MP_ROM_PTR(&pyb_wfi_obj) },
-    { MP_ROM_QSTR(MP_QSTR_sleep),               MP_ROM_PTR(&machine_sleep_obj) },
+    { MP_ROM_QSTR(MP_QSTR_sleep),               MP_ROM_PTR(&machine_lightsleep_obj) },
+    { MP_ROM_QSTR(MP_QSTR_lightsleep),          MP_ROM_PTR(&machine_lightsleep_obj) },
     { MP_ROM_QSTR(MP_QSTR_deepsleep),           MP_ROM_PTR(&machine_deepsleep_obj) },
     { MP_ROM_QSTR(MP_QSTR_reset_cause),         MP_ROM_PTR(&machine_reset_cause_obj) },
 #if 0
