@@ -27,6 +27,7 @@
 #include <stdio.h>
 
 #include "py/runtime.h"
+#include "extint.h"
 #include "rtc.h"
 #include "irq.h"
 
@@ -51,7 +52,7 @@ static mp_uint_t rtc_info;
 // Note: LSI is around (32KHz), these dividers should work either way
 // ck_spre(1Hz) = RTCCLK(LSE) /(uwAsynchPrediv + 1)*(uwSynchPrediv + 1)
 // modify RTC_ASYNCH_PREDIV & RTC_SYNCH_PREDIV in board/<BN>/mpconfigport.h to change sub-second ticks
-// default is 3906.25 µs, min is ~30.52 µs (will increas Ivbat by ~500nA)
+// default is 3906.25 us, min is ~30.52 us (will increase Ivbat by ~500nA)
 #ifndef RTC_ASYNCH_PREDIV
 #define RTC_ASYNCH_PREDIV (0x7f)
 #endif
@@ -60,11 +61,11 @@ static mp_uint_t rtc_info;
 #endif
 
 STATIC HAL_StatusTypeDef PYB_RTC_Init(RTC_HandleTypeDef *hrtc);
-STATIC void PYB_RTC_MspInit_Kick(RTC_HandleTypeDef *hrtc, bool rtc_use_lse);
+STATIC void PYB_RTC_MspInit_Kick(RTC_HandleTypeDef *hrtc, bool rtc_use_lse, bool rtc_use_byp);
 STATIC HAL_StatusTypeDef PYB_RTC_MspInit_Finalise(RTC_HandleTypeDef *hrtc);
 STATIC void RTC_CalendarConfig(void);
 
-#if defined(MICROPY_HW_RTC_USE_LSE) && MICROPY_HW_RTC_USE_LSE
+#if MICROPY_HW_RTC_USE_LSE || MICROPY_HW_RTC_USE_BYPASS
 STATIC bool rtc_use_lse = true;
 #else
 STATIC bool rtc_use_lse = false;
@@ -72,36 +73,6 @@ STATIC bool rtc_use_lse = false;
 STATIC uint32_t rtc_startup_tick;
 STATIC bool rtc_need_init_finalise = false;
 
-// check if LSE exists
-// not well tested, should probably be removed
-STATIC bool lse_magic(void) {
-#if 0
-    uint32_t mode_in = GPIOC->MODER & 0x3fffffff;
-    uint32_t mode_out = mode_in | 0x40000000;
-    GPIOC->MODER = mode_out;
-    GPIOC->OTYPER &= 0x7fff;
-    GPIOC->BSRRH = 0x8000;
-    GPIOC->OSPEEDR &= 0x3fffffff;
-    GPIOC->PUPDR &= 0x3fffffff;
-    int i = 0xff0;
-    __IO int d = 0;
-    uint32_t tc = 0;
-    __IO uint32_t j;
-    while (i) {
-        GPIOC->MODER = mode_out;
-        GPIOC->MODER = mode_in;
-        for (j = 0; j < d; j++) ;
-        i--;
-        if ((GPIOC->IDR & 0x8000) == 0) {
-            tc++;
-        }
-    }
-    return (tc < 0xff0)?true:false;
-#else
-    return false;
-#endif
-}
-
 void rtc_init_start(bool force_init) {
     RTCHandle.Instance = RTC;
 
@@ -123,7 +94,9 @@ void rtc_init_start(bool force_init) {
     rtc_need_init_finalise = false;
 
     if (!force_init) {
-        if ((RCC->BDCR & (RCC_BDCR_LSEON | RCC_BDCR_LSERDY)) == (RCC_BDCR_LSEON | RCC_BDCR_LSERDY)) {
+        uint32_t bdcr = RCC->BDCR;
+        if ((bdcr & (RCC_BDCR_RTCEN | RCC_BDCR_RTCSEL | RCC_BDCR_LSEON | RCC_BDCR_LSERDY))
+            == (RCC_BDCR_RTCEN | RCC_BDCR_RTCSEL_0 | RCC_BDCR_LSEON | RCC_BDCR_LSERDY)) {
             // LSE is enabled & ready --> no need to (re-)init RTC
             // remove Backup Domain write protection
             HAL_PWR_EnableBkUpAccess();
@@ -132,7 +105,8 @@ void rtc_init_start(bool force_init) {
             // provide some status information
             rtc_info |= 0x40000 | (RCC->BDCR & 7) | (RCC->CSR & 3) << 8;
             return;
-        } else if ((RCC->BDCR & RCC_BDCR_RTCSEL) == RCC_BDCR_RTCSEL_1) {
+        } else if ((bdcr & (RCC_BDCR_RTCEN | RCC_BDCR_RTCSEL))
+            == (RCC_BDCR_RTCEN | RCC_BDCR_RTCSEL_1)) {
             // LSI configured as the RTC clock source --> no need to (re-)init RTC
             // remove Backup Domain write protection
             HAL_PWR_EnableBkUpAccess();
@@ -148,14 +122,7 @@ void rtc_init_start(bool force_init) {
 
     rtc_startup_tick = HAL_GetTick();
     rtc_info = 0x3f000000 | (rtc_startup_tick & 0xffffff);
-    if (rtc_use_lse) {
-        if (lse_magic()) {
-            // don't even try LSE
-            rtc_use_lse = false;
-            rtc_info &= ~0x01000000;
-        }
-    }
-    PYB_RTC_MspInit_Kick(&RTCHandle, rtc_use_lse);
+    PYB_RTC_MspInit_Kick(&RTCHandle, rtc_use_lse, MICROPY_HW_RTC_USE_BYPASS);
 }
 
 void rtc_init_finalise() {
@@ -163,26 +130,34 @@ void rtc_init_finalise() {
         return;
     }
 
-    rtc_info = 0x20000000;
-    if (PYB_RTC_Init(&RTCHandle) != HAL_OK) {
+    rtc_info = 0;
+    while (PYB_RTC_Init(&RTCHandle) != HAL_OK) {
         if (rtc_use_lse) {
-            // fall back to LSI...
-            rtc_use_lse = false;
+            #if MICROPY_HW_RTC_USE_BYPASS
+            if (RCC->BDCR & RCC_BDCR_LSEBYP) {
+                // LSEBYP failed, fallback to LSE non-bypass
+                rtc_info |= 0x02000000;
+            } else
+            #endif
+            {
+                // LSE failed, fallback to LSI
+                rtc_use_lse = false;
+                rtc_info |= 0x01000000;
+            }
             rtc_startup_tick = HAL_GetTick();
-            PYB_RTC_MspInit_Kick(&RTCHandle, rtc_use_lse);
+            PYB_RTC_MspInit_Kick(&RTCHandle, rtc_use_lse, false);
             HAL_PWR_EnableBkUpAccess();
             RTCHandle.State = HAL_RTC_STATE_RESET;
-            if (PYB_RTC_Init(&RTCHandle) != HAL_OK) {
-                rtc_info = 0x0100ffff; // indicate error
-                return;
-            }
         } else {
             // init error
-            rtc_info = 0xffff; // indicate error
+            rtc_info |= 0xffff; // indicate error
             return;
         }
     }
 
+    // RTC started successfully
+    rtc_info = 0x20000000;
+
     // record if LSE or LSI is used
     rtc_info |= (rtc_use_lse << 28);
 
@@ -253,6 +228,16 @@ STATIC HAL_StatusTypeDef PYB_RCC_OscConfig(RCC_OscInitTypeDef  *RCC_OscInitStruc
         }
         #endif
 
+        #if MICROPY_HW_RTC_USE_BYPASS
+        // If LSEBYP is enabled and new state is non-bypass then disable LSEBYP
+        if (RCC_OscInitStruct->LSEState == RCC_LSE_ON && (RCC->BDCR & RCC_BDCR_LSEBYP)) {
+            CLEAR_BIT(RCC->BDCR, RCC_BDCR_LSEON);
+            while (RCC->BDCR & RCC_BDCR_LSERDY) {
+            }
+            CLEAR_BIT(RCC->BDCR, RCC_BDCR_LSEBYP);
+        }
+        #endif
+
         // Set the new LSE configuration
         __HAL_RCC_LSE_CONFIG(RCC_OscInitStruct->LSEState);
     }
@@ -323,7 +308,7 @@ STATIC HAL_StatusTypeDef PYB_RTC_Init(RTC_HandleTypeDef *hrtc) {
     }
 }
 
-STATIC void PYB_RTC_MspInit_Kick(RTC_HandleTypeDef *hrtc, bool rtc_use_lse) {
+STATIC void PYB_RTC_MspInit_Kick(RTC_HandleTypeDef *hrtc, bool rtc_use_lse, bool rtc_use_byp) {
     /* To change the source clock of the RTC feature (LSE, LSI), You have to:
        - Enable the power clock using __PWR_CLK_ENABLE()
        - Enable write access using HAL_PWR_EnableBkUpAccess() function before to
@@ -338,12 +323,14 @@ STATIC void PYB_RTC_MspInit_Kick(RTC_HandleTypeDef *hrtc, bool rtc_use_lse) {
     RCC_OscInitTypeDef RCC_OscInitStruct;
     RCC_OscInitStruct.OscillatorType =  RCC_OSCILLATORTYPE_LSI | RCC_OSCILLATORTYPE_LSE;
     RCC_OscInitStruct.PLL.PLLState = RCC_PLL_NONE;
-    if (rtc_use_lse) {
-        #if MICROPY_HW_RTC_USE_BYPASS
+    #if MICROPY_HW_RTC_USE_BYPASS
+    if (rtc_use_byp) {
         RCC_OscInitStruct.LSEState = RCC_LSE_BYPASS;
-        #else
+        RCC_OscInitStruct.LSIState = RCC_LSI_OFF;
+    } else
+    #endif
+    if (rtc_use_lse) {
         RCC_OscInitStruct.LSEState = RCC_LSE_ON;
-        #endif
         RCC_OscInitStruct.LSIState = RCC_LSI_OFF;
     } else {
         RCC_OscInitStruct.LSEState = RCC_LSE_OFF;
@@ -355,16 +342,29 @@ STATIC void PYB_RTC_MspInit_Kick(RTC_HandleTypeDef *hrtc, bool rtc_use_lse) {
     rtc_need_init_finalise = true;
 }
 
-#define PYB_LSE_TIMEOUT_VALUE 1000  // ST docs spec 2000 ms LSE startup, seems to be too pessimistic
-#define PYB_LSI_TIMEOUT_VALUE 500   // this is way too pessimistic, typ. < 1ms
+#ifndef MICROPY_HW_RTC_LSE_TIMEOUT_MS
+#define MICROPY_HW_RTC_LSE_TIMEOUT_MS 1000  // ST docs spec 2000 ms LSE startup, seems to be too pessimistic
+#endif
+#ifndef MICROPY_HW_RTC_LSI_TIMEOUT_MS
+#define MICROPY_HW_RTC_LSI_TIMEOUT_MS 500   // this is way too pessimistic, typ. < 1ms
+#endif
+#ifndef MICROPY_HW_RTC_BYP_TIMEOUT_MS
+#define MICROPY_HW_RTC_BYP_TIMEOUT_MS 150
+#endif
 
 STATIC HAL_StatusTypeDef PYB_RTC_MspInit_Finalise(RTC_HandleTypeDef *hrtc) {
     // we already had a kick so now wait for the corresponding ready state...
     if (rtc_use_lse) {
         // we now have to wait for LSE ready or timeout
+        uint32_t timeout = MICROPY_HW_RTC_LSE_TIMEOUT_MS;
+        #if MICROPY_HW_RTC_USE_BYPASS
+        if (RCC->BDCR & RCC_BDCR_LSEBYP) {
+            timeout = MICROPY_HW_RTC_BYP_TIMEOUT_MS;
+        }
+        #endif
         uint32_t tickstart = rtc_startup_tick;
         while (__HAL_RCC_GET_FLAG(RCC_FLAG_LSERDY) == RESET) {
-            if ((HAL_GetTick() - tickstart ) > PYB_LSE_TIMEOUT_VALUE) {
+            if ((HAL_GetTick() - tickstart ) > timeout) {
                 return HAL_TIMEOUT;
             }
         }
@@ -372,7 +372,7 @@ STATIC HAL_StatusTypeDef PYB_RTC_MspInit_Finalise(RTC_HandleTypeDef *hrtc) {
         // we now have to wait for LSI ready or timeout
         uint32_t tickstart = rtc_startup_tick;
         while (__HAL_RCC_GET_FLAG(RCC_FLAG_LSIRDY) == RESET) {
-            if ((HAL_GetTick() - tickstart ) > PYB_LSI_TIMEOUT_VALUE) {
+            if ((HAL_GetTick() - tickstart ) > MICROPY_HW_RTC_LSI_TIMEOUT_MS) {
                 return HAL_TIMEOUT;
             }
         }
@@ -439,7 +439,7 @@ STATIC mp_obj_t pyb_rtc_make_new(const mp_obj_type_t *type, size_t n_args, size_
     mp_arg_check_num(n_args, n_kw, 0, 0, false);
 
     // return constant object
-    return (mp_obj_t)&pyb_rtc_obj;
+    return MP_OBJ_FROM_PTR(&pyb_rtc_obj);
 }
 
 // force rtc to re-initialise
@@ -529,7 +529,6 @@ mp_obj_t pyb_rtc_datetime(size_t n_args, const mp_obj_t *args) {
         time.Hours = mp_obj_get_int(items[4]);
         time.Minutes = mp_obj_get_int(items[5]);
         time.Seconds = mp_obj_get_int(items[6]);
-        time.SubSeconds = rtc_us_to_subsec(mp_obj_get_int(items[7]));
         time.TimeFormat = RTC_HOURFORMAT12_AM;
         time.DayLightSaving = RTC_DAYLIGHTSAVING_NONE;
         time.StoreOperation = RTC_STOREOPERATION_SET;
@@ -540,6 +539,10 @@ mp_obj_t pyb_rtc_datetime(size_t n_args, const mp_obj_t *args) {
 }
 MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(pyb_rtc_datetime_obj, 1, 2, pyb_rtc_datetime);
 
+#if defined(STM32F0)
+#define RTC_WKUP_IRQn RTC_IRQn
+#endif
+
 // wakeup(None)
 // wakeup(ms, callback=None)
 // wakeup(wucksel, wut, callback)
@@ -609,17 +612,17 @@ mp_obj_t pyb_rtc_wakeup(size_t n_args, const mp_obj_t *args) {
     }
 
     // set the callback
-    MP_STATE_PORT(pyb_extint_callback)[22] = callback;
+    MP_STATE_PORT(pyb_extint_callback)[EXTI_RTC_WAKEUP] = callback;
 
     // disable register write protection
     RTC->WPR = 0xca;
     RTC->WPR = 0x53;
 
     // clear WUTE
-    RTC->CR &= ~(1 << 10);
+    RTC->CR &= ~RTC_CR_WUTE;
 
     // wait until WUTWF is set
-    while (!(RTC->ISR & (1 << 2))) {
+    while (!(RTC->ISR & RTC_ISR_WUTWF)) {
     }
 
     if (enable) {
@@ -634,26 +637,26 @@ mp_obj_t pyb_rtc_wakeup(size_t n_args, const mp_obj_t *args) {
         // enable register write protection
         RTC->WPR = 0xff;
 
-        // enable external interrupts on line 22
+        // enable external interrupts on line EXTI_RTC_WAKEUP
         #if defined(STM32L4)
-        EXTI->IMR1 |= 1 << 22;
-        EXTI->RTSR1 |= 1 << 22;
+        EXTI->IMR1 |= 1 << EXTI_RTC_WAKEUP;
+        EXTI->RTSR1 |= 1 << EXTI_RTC_WAKEUP;
         #elif defined(STM32H7)
-        EXTI_D1->IMR1 |= 1 << 22;
-        EXTI->RTSR1   |= 1 << 22;
+        EXTI_D1->IMR1 |= 1 << EXTI_RTC_WAKEUP;
+        EXTI->RTSR1 |= 1 << EXTI_RTC_WAKEUP;
         #else
-        EXTI->IMR |= 1 << 22;
-        EXTI->RTSR |= 1 << 22;
+        EXTI->IMR |= 1 << EXTI_RTC_WAKEUP;
+        EXTI->RTSR |= 1 << EXTI_RTC_WAKEUP;
         #endif
 
         // clear interrupt flags
-        RTC->ISR &= ~(1 << 10);
+        RTC->ISR &= ~RTC_ISR_WUTF;
         #if defined(STM32L4)
-        EXTI->PR1 = 1 << 22;
+        EXTI->PR1 = 1 << EXTI_RTC_WAKEUP;
         #elif defined(STM32H7)
-        EXTI_D1->PR1 = 1 << 22;
+        EXTI_D1->PR1 = 1 << EXTI_RTC_WAKEUP;
         #else
-        EXTI->PR = 1 << 22;
+        EXTI->PR = 1 << EXTI_RTC_WAKEUP;
         #endif
 
         NVIC_SetPriority(RTC_WKUP_IRQn, IRQ_PRI_RTC_WKUP);
@@ -662,18 +665,18 @@ mp_obj_t pyb_rtc_wakeup(size_t n_args, const mp_obj_t *args) {
         //printf("wut=%d wucksel=%d\n", wut, wucksel);
     } else {
         // clear WUTIE to disable interrupts
-        RTC->CR &= ~(1 << 14);
+        RTC->CR &= ~RTC_CR_WUTIE;
 
         // enable register write protection
         RTC->WPR = 0xff;
 
-        // disable external interrupts on line 22
+        // disable external interrupts on line EXTI_RTC_WAKEUP
         #if defined(STM32L4)
-        EXTI->IMR1 &= ~(1 << 22);
+        EXTI->IMR1 &= ~(1 << EXTI_RTC_WAKEUP);
         #elif defined(STM32H7)
-        EXTI_D1->IMR1 |= 1 << 22;
+        EXTI_D1->IMR1 |= 1 << EXTI_RTC_WAKEUP;
         #else
-        EXTI->IMR &= ~(1 << 22);
+        EXTI->IMR &= ~(1 << EXTI_RTC_WAKEUP);
         #endif
     }
 
