@@ -37,8 +37,6 @@
 
 #if MICROPY_PY_SYS
 
-#include "genhdr/mpversion.h"
-
 // defined per port; type of these is irrelevant, just need pointer
 extern struct _mp_dummy_t mp_sys_stdin_obj;
 extern struct _mp_dummy_t mp_sys_stdout_obj;
@@ -106,7 +104,8 @@ STATIC mp_obj_t mp_sys_print_exception(size_t n_args, const mp_obj_t *args) {
     #if MICROPY_PY_IO && MICROPY_PY_SYS_STDFILES
     void *stream_obj = &mp_sys_stdout_obj;
     if (n_args > 1) {
-        stream_obj = MP_OBJ_TO_PTR(args[1]); // XXX may fail
+        mp_get_stream_raise(args[1], MP_STREAM_OP_WRITE);
+        stream_obj = MP_OBJ_TO_PTR(args[1]);
     }
 
     mp_print_t print = {stream_obj, mp_stream_write_adaptor};
