@@ -26,11 +26,15 @@
 #ifndef MICROPY_INCLUDED_PY_RUNTIME0_H
 #define MICROPY_INCLUDED_PY_RUNTIME0_H
 
-// These must fit in 8 bits; see scope.h
+// The first four must fit in 8 bits, see emitbc.c
+// The remaining must fit in 16 bits, see scope.h
 #define MP_SCOPE_FLAG_VARARGS      (0x01)
 #define MP_SCOPE_FLAG_VARKEYWORDS  (0x02)
 #define MP_SCOPE_FLAG_GENERATOR    (0x04)
 #define MP_SCOPE_FLAG_DEFKWARGS    (0x08)
+#define MP_SCOPE_FLAG_REFGLOBALS   (0x10) // used only if native emitter enabled
+#define MP_SCOPE_FLAG_HASCONSTS    (0x20) // used only if native emitter enabled
+#define MP_SCOPE_FLAG_VIPERRET_POS    (6) // 3 bits used for viper return type
 
 // types for native (viper) function signature
 #define MP_NATIVE_TYPE_OBJ  (0x00)
@@ -57,6 +61,7 @@ typedef enum {
     MP_UNARY_OP_LEN, // __len__
     MP_UNARY_OP_HASH, // __hash__; must return a small int
     MP_UNARY_OP_ABS, // __abs__
+    MP_UNARY_OP_INT, // __int__
     MP_UNARY_OP_SIZEOF, // for sys.getsizeof()
 
     MP_UNARY_OP_NUM_RUNTIME,
@@ -143,8 +148,12 @@ typedef enum {
 } mp_binary_op_t;
 
 typedef enum {
-    MP_F_CONVERT_OBJ_TO_NATIVE = 0,
+    MP_F_CONST_NONE_OBJ = 0,
+    MP_F_CONST_FALSE_OBJ,
+    MP_F_CONST_TRUE_OBJ,
+    MP_F_CONVERT_OBJ_TO_NATIVE,
     MP_F_CONVERT_NATIVE_TO_OBJ,
+    MP_F_NATIVE_SWAP_GLOBALS,
     MP_F_LOAD_NAME,
     MP_F_LOAD_GLOBAL,
     MP_F_LOAD_BUILD_CLASS,
@@ -164,8 +173,8 @@ typedef enum {
     MP_F_BUILD_MAP,
     MP_F_STORE_MAP,
 #if MICROPY_PY_BUILTINS_SET
-    MP_F_BUILD_SET,
     MP_F_STORE_SET,
+    MP_F_BUILD_SET,
 #endif
     MP_F_MAKE_FUNCTION_FROM_RAW_CODE,
     MP_F_NATIVE_CALL_FUNCTION_N_KW,
@@ -188,12 +197,14 @@ typedef enum {
     MP_F_DELETE_GLOBAL,
     MP_F_NEW_CELL,
     MP_F_MAKE_CLOSURE_FROM_RAW_CODE,
+    MP_F_ARG_CHECK_NUM_SIG,
     MP_F_SETUP_CODE_STATE,
     MP_F_SMALL_INT_FLOOR_DIVIDE,
     MP_F_SMALL_INT_MODULO,
+    MP_F_NATIVE_YIELD_FROM,
     MP_F_NUMBER_OF,
 } mp_fun_kind_t;
 
-extern void *const mp_fun_table[MP_F_NUMBER_OF];
+extern const void *const mp_fun_table[MP_F_NUMBER_OF];
 
 #endif // MICROPY_INCLUDED_PY_RUNTIME0_H
