@@ -15,3 +15,10 @@ try:
 except AttributeError:
     print('SKIP')
     raise SystemExit
+
+# __name__ of a bound native method is not implemented in uPy
+# the test here is to make sure it doesn't crash
+try:
+    str((1).to_bytes.__name__)
+except AttributeError:
+    pass
