@@ -1,13 +1,16 @@
 try:
     import ujson as json
 except ImportError:
-    import json
+    try:
+        import json
+    except ImportError:
+        print("SKIP")
+        raise SystemExit
 
 print(json.dumps(False))
 print(json.dumps(True))
 print(json.dumps(None))
 print(json.dumps(1))
-print(json.dumps(1.2))
 print(json.dumps('abc'))
 print(json.dumps('\x00\x01\x7e'))
 print(json.dumps([]))
