@@ -1,8 +1,15 @@
 import pyb
 from pyb import I2C
 
+# test we can correctly create by id or name
+for bus in (-1, 0, 1, 2, 3, "X", "Y", "Z"):
+    try:
+        I2C(bus)
+        print("I2C", bus)
+    except ValueError:
+        print("ValueError", bus)
+
 i2c = I2C(1)
-i2c2 = I2C(2)
 
 i2c.init(I2C.MASTER, baudrate=400000)
 print(i2c.scan())
@@ -12,7 +19,8 @@ i2c.deinit()
 
 accel_addr = 76
 
-pyb.Accel() # this will init the bus for us
+pyb.Accel() # this will init the MMA for us
+i2c.init(I2C.MASTER, baudrate=400000)
 
 print(i2c.scan())
 print(i2c.is_ready(accel_addr))
